<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */

/**
 * @purpose FoFRedux API for handling all program logic. This file is the "controller"
 * of an MVC design.
 *
 * @version $Revision: 545 $
 * $Id: init.php 545 2007-02-24 08:35:36Z kevbob $
 */

require_once "config.php";
$FOF_NONSECURE_VIEW = true;
// set version of application
define('FOF_VERSION', 'RC 0.4 (ui 2.0)');

if (!defined('FOF_PHP_DEBUG')) {
	define('FOF_PHP_DEBUG', false);
}

define('FOF_ERROR_LOG', "/tmp/foferr.log");

if (FOF_PHP_DEBUG == true) {
  
	error_reporting(E_ALL);
   // setup debugging
   #set_error_handler("fof_logfile_handler");

} else {
	error_reporting(E_ALL ^ E_NOTICE);
}

// load model
require_once FOF_DIR . 'lib/model/base.php';


// load presentation functions/classes
require_once FOF_DIR . 'lib/view_helper.php';
require_once FOF_DIR . 'lib/view.class.php';

define('FOF_VIEW_DIR', FOF_DIR . '/view');

if ( !isset($FOF_INSTALLATION_ACTIVE) ) {
   $opt_model = FOF_ModelFactory::getOptionsModel();
   $opts = $opt_model->fetch();
   unset($opt_model);

   if ($opts->get('global.gzip_output') == 1)
      ob_start("ob_gzhandler");
}

// auto-detect base URI and URL for this installation
define('FOF_BASE_URI', dirname($_SERVER["SCRIPT_NAME"]) . '/');

if (isset($_SERVER['HTTPS']) and $_SERVER['HTTPS'] = 'on') {
   define('FOF_BASE_URL', sprintf('https://%s%s', $_SERVER["HTTP_HOST"], FOF_BASE_URI));
}
else {
   define('FOF_BASE_URL', sprintf('http://%s%s', $_SERVER["HTTP_HOST"], FOF_BASE_URI));
}



// setup session and redirect to login page if necessary
//session_start();

if ( !isset($FOF_NONSECURE_VIEW)
     or ($FOF_NONSECURE_VIEW != true and !$_SESSION['username']) ) {

   header("Location: login.php");
   exit;
}




function fof_check_login($username, $entered_pass)
{
/****
 * 23.04.06 / eroth
 * 
 * this will essentially authenticate a user and store it in the session
 * returning "OK" for success, else an error text
 * @param string $username
 * @param string $entered_pass
*/
   $user_model = FOF_ModelFactory::getUserModel();

   $user = $user_model->findUser( $username );

   if ($user == false) {  //no record for this user found, so no pass retrieved
      return "Login Error:  User ($username) not found in the database.";
   }
   else {
      //figure out which encryption we have and grab the salt
      $stored_pass = $user->getPassword();
      
      $salt_sub = substr($stored_pass,0,3);
      if ($salt_sub=="$2$"){  //blowfish
         $salt=substr($stored_pass,0,16);
      }
      elseif ($salt_sub=="$1$"){ //MD5
         $salt=substr($stored_pass,0,12);
      }
      else{   //DES
         $salt=substr($stored_pass,0,2);
      }
      if (crypt($entered_pass,$salt) != $stored_pass){
         return "Login Error:  Password authentication failed.";
      }
      else {
         return "OK";
      }
   }
}

/**
 * This is to add a clone() function to php4 
 * the function is included in php5 and above 
 * 
 */
if (version_compare(phpversion(), '5.0') < 0) {
   eval('
      function clone($object) {
         return $object;
      }
       ');
}


/**
 * parse opml file and return an array  of urls and categories if set
 *
 * @todo nilspace - make the regexp more robust for getting the category
 *
 * @param string $opml
 * @return array
 */
function fof_opml_to_array($opml)
{
   $rx = "/xmlurl=\"(.*?)\"( category=\"(.*?)\")?/mi";

	$resultArray = array();
	$matches = null;
   if (preg_match_all($rx, $opml, $matches))
   {
      for($i = 0; $i < count($matches[0]) ; $i++)
      {
         $resultArray[$i][0] = $matches[1][$i];
				 if($matches[2][$i] != "")
					 $resultArray[$i][1] = $matches[3][$i];
				else
						$resultArray[$i][1] = "None";

      }
  }

  return $resultArray;
}

/**
 * construct and return a common header for all html pages
 * 
 * @param void
 * @return View
 * 
 */
function fof_getHeaderView($additional_markup = null)
{
  global $title;
  global $view;
	
	@header("Content-Type: text/html; charset=utf-8");
	
	
	if(isset($_GET['view']))
	    $view = $_GET['view'];
	    
	// seperate "if" statement in case this file was included
	//  with another php file that already defined "view"
	if(!isset($view))
	    $view = "normal";
	
	if (!isset($title)) {
		$title = 'fofredux';
	}
	
	$header_view = new View( FOF_VIEW_DIR . DIR_SEP . '_header.php' );
	$header_view->set('view', $view);
	$header_view->set('title', $title);
	$header_view->set('add_head', (!is_null($additional_markup)) ? $additional_markup : '');

	return $header_view;
  
}

/**
 * construct and return a common footer used on all html pages
 * 
 */
function fof_getFooterView()
{
  $footer_view = new View( FOF_VIEW_DIR . DIR_SEP . '_footer.php' );
  return $footer_view;
}

function fof_getLayoutView($main_view, $additional_markup = null)
{
  $view = new LayoutView(fof_getHeaderView($additional_markup),
                         fof_getFooterView(),
                         $main_view);
  return $view;
}

/**
 *
 * @param string $url
 * @param string $category
 * @return void
 */
function fof_add_feed($url,$category_id)
{
   if(!$url) return;

   $url = trim($url);

   // category
   $cat_model = FOF_ModelFactory::getCategoryModel();
   if(is_null($category_id)) {
      $category = $cat_model->fetchDefaultCategory();
   }
   else {
      $category = $cat_model->fetch((int)$category_id);
   }

   $feed_model = FOF_ModelFactory::getFeedModel();
   
	
   if(substr($url, 0, 7) != 'http://' && substr($url, 0, 8) != 'https://')
   {
     $url = 'http://' . $url;
   }

   $s = "<p class='message'><span>Attempting to subscribe to <a href=\"$url\">$url</a>...<br>";

   if($feed = $feed_model->findFeed($url))
   {
      $s .= "You are already subscribed to " . fof_render_feed_link($feed) . "</span></p>";
	  $s = str_replace('message', 'message warning',$s);
	  print $s;
      return true;
   }

   $rss_model = FOF_ModelFactory::getRssModel();
   $rssFeed = $rss_model->getRssFeed( $url, true );


   if ( !$rssFeed ) {
      $s .= "&nbsp;&nbsp;URL is not RSS or is invalid.<br>";
      $s .= "&nbsp;&nbsp;(error was: <B>" . magpie_error() . "</b>)<br>";
      $s .= "&nbsp;&nbsp;<a href=\"http://feeds.archive.org/validator/check?url=$url\">The validator may give more information.</a><br>";

      $s .= "<br>Attempting autodiscovery...<br><br>";

      $r = _fetch_remote_file ($url);
      $c = $r->results;

      if($c && $r->status >= 200 && $r->status < 300)
      {
         $discoveredUrl = fof_getRSSLocation($c, $url);
         if($discoveredUrl)
         {
            $s .= "Autodiscovery found <a href=\"$discoveredUrl\">$discoveredUrl</a>.<br>";

            $s .= "Attempting to subscribe to <a href=\"$discoveredUrl\">$discoveredUrl</a>...<br>";

            if($feed = $feed_model->findFeed($discoveredUrl))
            {
               $s .= "<br>You are already subscribed to " . fof_render_feed_link($feed) . "</span></p>";
			   $s = str_replace('message', 'message warning',$s);
			   print $s;
               return true;
            }

            $rssFeed = $rss_model->getRssFeed( $discoveredUrl, true );

            if(!$rssFeed)
            {
               $s .= "&nbsp;&nbsp;URL is not RSS, giving up.<br>";
               $s .= "&nbsp;&nbsp;(error was: <strong>" . $rss_model->error() . "</strong>)<br>";
               $s .= "&nbsp;&nbsp;<a href=\"http://feeds.archive.org/validator/check?url=$discoveredUrl\">The validator may give more information.</a></span></p>";
		   	  $status = "success";

            }
            else
            {

               $val = fof_actually_add_feed($rssFeed, $category);
               if ($val) {
                  $s .= "&nbsp;&nbsp;Subscribed.";
			   	  $status = "success";
               }
               else {
                  $s .= "Error Subscribing to feed <br>";
			   	  $status = "error";
               }
            }
         }
         else
         {
            $s .= "Autodiscovery failed.  Giving up.<br>";
	   	 	$status = "error";
         }
      }
      else
      {
         $s .= "Can't load URL.  Giving up.<br>";
	   	 $status = "error";
      }
   }
   else
   {
      $val = fof_actually_add_feed($rssFeed, $category);
      if ($val) {
         $s .= "Subscribed. (category: <a href='view.php?category=".$category->getId()."' title='Feeds marked with category: ".$category->getTitle()."'>".$category->getTitle()."</a>)<br>";
	   	 $status = "success";
      }
      else {
         $s .= "Error Subscribing to feed <br>";
	   	 $status = "error";
      }
   }
   $s .= "</p>";
   $s = str_replace('message', 'message '.$status,$s);
   echo $s;
}


/**
 * 
 * 
 * @param string $url
 * @param array $rss
 * @param FOF_Category $category
 */
function fof_actually_add_feed($rssFeed, $category)
{

   $feed_model = FOF_ModelFactory::getFeedModel();

   $url = $rssFeed->getUrl();
   $title = $rssFeed->getTitle();
   $link = $rssFeed->getLink();

   // can't be null in db.  set to empty string instead.
   if (is_null($link)) {
   	$link = '';
   }

   $feed = $feed_model->create();
   $feed->setUrl($url);
   $feed->setTitle($title);
   $feed->setLink($link);
   $feed->setCategory($category);
   $feed->setAutoTag(false);
   $feed->setUpdateEnabled(true);
   
   return $feed_model->save($feed);
}


/**
 * Retreive and insert new items from feed. purge old
 * items if FOF_KEEP_DAYS is set.
 *
 * @param FOF_Feed $feed
 * @return int  count of new items inserted  (-1 = bad input, -2 = updates disabled)
 */
function fof_update_feed($feed)
{
   global $opts;

   $feed_model = FOF_ModelFactory::getFeedModel();
   
	if (!$feed->isUpdateEnabled()) {
		return -2;
	}

   $rss_model = FOF_ModelFactory::getRssModel();
   $rss_feed = $rss_model->getRssFeed($feed->getUrl(), true);

   if(!$rss_feed)
   {
      print "Error: <B>" . $rss_model->error() . "</b> ";
      print "<a href=\"http://feeds.archive.org/validator/check?url=$url\">try to validate it?</a> ";
      return 0;
   }
   
   $feed->setLastRssUpdate(time());
   $feed_model->save($feed);

   $rss_items = $rss_feed->getItems();
   $rss_items->rewind();

   $itemsInserted = 0;
   $items_processed = array();

   while ($rss_items->valid())
   {
      $rss_item = $rss_items->current();
      $rss_items->next();
      
		list($item, $inserted) = fof_add_item($rss_item, $feed);


		if ( $inserted == true) {
			$itemsInserted += 1;
		}
		$items_processed[] = $item->getId();

		
		if ( $feed->isAutoTag() ) {
			//collect list of tags for item
			// TODO add tags to feed item once we have the list
      }

   }

	// remove old items
   if ($opts->get('global.keep_days') > 0) {
		fof_feed_purge($feed, $opts->get('global.keep_days'), $items_processed);

   }

   return $itemsInserted;
}


/**
 * purge items from database
 * 
 * @param FOF_Feed $feed
 * @param int $keep_days
 * @param array $keep_items array of item ids
 * @return int number of items deleted
 */
function fof_feed_purge($feed, $keep_days, $keep_items)
{

   $item_model = FOF_ModelFactory::getItemModel();
   $item_search = new FOF_ItemSearch();
   $item_search->setOffset(0);
   $item_search->setLimit(0);
   $item_search->setSortField('i.id');
   
   $item_search->setFeed((int)$feed->getId());
   
   // exclude items that still exist in the RSS feed
   if (count($keep_items) > 0) {
      $item_search->addFilter(new FOF_NotInFilter('i.id', $keep_items));
   }
   
   // calculate the timestamp where older items should be removed
   $date_ts = time() - (86400 * $keep_days);
   
   $dateFilter = new FOF_LessThanFilter('cache_time', $date_ts);
   $dateFilter->setFormatter(new FOF_DBTimeFormatter());
   $item_search->addFilter($dateFilter);

   // add filter to only include read items.
   //  (which implicitly excludes unread and saved items)
   $item_search->setFlag(FOF_READ_FLAG);
   
   $item_iterator = $item_model->search($item_search);
   $item_iterator->rewind();

   $counter = 0;
   while ($item_iterator->valid()) {
      $item = $item_iterator->current();
      $item_iterator->next();
      
      $item_model->remove($item);
      $counter++;
   }
   return $counter;
}


/**
 * add an item to a feed.  The item information is normalized to fit into 
 * the database
 *
 * @param FOF_RssItem $rss_item
 * @param FOF_Feed $feed
 * @return array (<FOF_Item>, <inserted flag>)
 */
function fof_add_item($rss_item, $feed)
{

   $item_model = FOF_ModelFactory::getItemModel();
   
   $item = $item_model->findItem($rss_item->getUniqueHash());
   
   if ($item) {
      return array($item, false);
   }
   
   $item = $item_model->create();
   
   // transfer values from rss item to db item
   $item->setLink( $rss_item->getLink());
   $item->setFeed($feed);
   $item->setTitle($rss_item->getTitle());
   $item->setContent($rss_item->getContent());
   $item->setPublishTime($rss_item->getPublishTime());
   $item->setCacheTime($rss_item->getCacheTime());
   $item->setCreator($rss_item->getCreator());
   $item->setFlag(FOF_UNREAD_FLAG);
   $item->setLatitude($rss_item->getLatitude());
   $item->setLongitude($rss_item->getLongitude());
   $item->setEnclosureLink($rss_item->getEnclosureLink());
   $item->setEnclosureType($rss_item->getEnclosureType());
   $item->setHash( $rss_item->getUniqueHash() );
   
   $item_model->save($item);

	return array($item, true );
}


function fof_logfile_handler ($errno, $errstr, $errfile, $errline) {
  $myerr = "ERROR $errno errfile $errfile errline $errline errstr $errstr";

  $handle = fopen (FOF_ERROR_LOG, "a");
  $timestamp = date("F j G:i:s");
  fwrite ($handle, $timestamp." ".$myerr."\n");
  fclose ($handle);

}

