<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * fofRedux Copyright (C) 2005 Andrew Turner
 * nilspace@users.sourceforge.net - http://highearthorbit.com
 * 
 * Distributed under the GPL - see LICENSE
 *
 */

/**
 * @purpose installs or updates an FoFRedux or original FeedOnFeeds database.
 * @version $Revision: 521 $
 * $Id: install.php 521 2006-10-04 02:21:13Z kevbob $
 * 
 * The installer currently generates tables based on the v0.19 release of feedonfeeds, 
 * and then performs tests and updates to these tables for upgrading to FoFRedux.
 *
 */

// flag to indicate no initial database access should be made
$FOF_INSTALLATION_ACTIVE = true;

// flag to indicate no initial authentication should be attempted
$FOF_NONSECURE_VIEW = true;

require_once "header.php";
require_once 'lib/install_helper.php';


/****
 * do our initial checking here...
 * if we are ok, then set $installing to true and go on with install/update
 * 
 * note...not really important for user stuff, since that is validated by javascript,
 * but it'll be essential when create config.php from this script.  /eroth
 */
if (isset($_POST['email']) && isset($_POST['password']) && isset($_POST['username'])){
	$username = $_POST['username'];
	$pass = $_POST['password'];
	$email = $_POST['email'];
}
else {

   // check to see if admin user is already created (update instead of install)

   $form_view = new View(FOF_VIEW_DIR . DIR_SEP . '_install_form.php');
   $form_view->render();
   
   include("footer.php");
   exit;
}

	
?>
<h1 style="font-size:250%;padding:30px;">Feed on Feeds Redux Installer</h1>
<?php

$log = array();

// compile list of models to install
$models = array();
   
$models[] = FOF_ModelFactory::getFeedModel();
$models[] = FOF_ModelFactory::getUserModel();
$models[] = FOF_ModelFactory::getCategoryModel();
$models[] = FOF_ModelFactory::getOptionsModel();
$models[] = FOF_ModelFactory::getItemModel();
$models[] = FOF_ModelFactory::getTagModel();

   
// loop through each model, running the install method of each
foreach ($models as $model) {
   	
   $commands = $model->install();
   $commands->rewind();

   // the install method returns and iterator of command objects
   // execute each command, checking if it was successful
   while ( $commands->valid() ) {
      $cmd = $commands->current();
      $commands->next();
      
      $resultFlag = $cmd->execute();
      $text = $cmd->getMessage();
      $log[] = array($text, $resultFlag);
   }
}



/*
 * 22.04.06 / eroth
 * add the first user as admin, based on his input before the submit
 * we'll also encrypt the password using the best available method
 */


$user_model = FOF_ModelFactory::getUserModel();
$users = $user_model->getUsers();
$users->rewind();

if (!$users->valid()) {
   $salt = md5(microtime() . getmypid());
   $cryptopass = ""; 
   if (CRYPT_BLOWFISH == 1) {
         $cryptopass = crypt($pass, '$2$'.substr($salt, 0, 13));
   } elseif (CRYPT_MD5 == 1) {

         $cryptopass = crypt($pass, '$1$'.substr($salt, 0, 8).'$');
   } else {  //CRYPT_STD_DES

         $cryptopass = crypt($pass, substr($salt, 0, 2));
   }

   $user = $user_model->create();
   
   $user->setUsername($username);
   $user->setPassword($cryptopass);
   $user->setEmail($email);
   $user->setAdmin(true);
   
   $user_model->save($user);

   $log[] = array("add admin user: added '$username', as an admin", true);   

}
else {
	$log[] = array("add admin user: at least 1 user already exists.", true);   
   
}

// run non-database install commands these are contained in lib/install_helper.php
//
$otherCommands = fof_getGenericInstallCommands();
$otherCommands->rewind();

while ( $otherCommands->valid() ) {
	
   $cmd = $otherCommands->current();
   $otherCommands->next();

   $resultFlag = $cmd->execute();
   $text = $cmd->getMessage();
   $log[] = array($text, $resultFlag);

}

// display log messages in a view
$result_table_view = new View(FOF_VIEW_DIR . DIR_SEP . '_install_table.php');
$result_table_view->set('log', $log);
$result_table_view->render();

?>
<br/>
<br/>
	Setup complete! <a href="index.php">Go to the control panel and start subscribing.</a>
<?php

include("footer.php");
