<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose provide base functionality for all model classes
 * @version $Revision: 296 $
 * $Id: base.php 511 2006-08-27 16:53:45Z nilspace $
 */


define('ADODB_ERROR_HANDLER', 'fof_db_error_handler');

require_once FOF_DIR . 'adodb_lite/adodb.inc.php';
require_once FOF_DIR . 'adodb_lite/adodb-errorhandler.inc.php';


if ( !defined('FOF_DB_TYPE') ) {
   define('FOF_DB_TYPE', "mysql");
   echo "missing FOF_DB_TYPE definition in config.php, please add one. See config.php.sample<br>\n";
}

if ( !defined('FOF_DB_DEBUG')) {
   define('FOF_DB_DEBUG', false);
}

// always use ASSOC fetch mode for the database
$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC; // DEFAULT, NUM, ASSOC or BOTH. Default follows native driver default...


// define global to hold singleton database connection
$GLOBALS["FOF_DB_CONNECTION"] = null;


class FOF_BaseModel
{
   var $user;
   
   /**
    * base constructor for model
    * 
    * @param ADOConnection $db
    */
   function FOF_BaseModel()
   {
   }

   /**
    * store user in model instance
    * the user is very important because all database operations will
    * require a valid user
    * 
    * @param FOF_User $user
    */
   function setUser( $user )
   {
      $this->user = $user;
   }

   /**
    * fetch the current database connection.  This should be
    * used by all that need the current db connection, both internal and external
    * 
    * @param void
    * @return ADOConnection
    * @access protected
    */
   function getDB()
   {
      if ( is_null($GLOBALS["FOF_DB_CONNECTION"]) ) {
         $GLOBALS["FOF_DB_CONNECTION"] = FOF_BaseModel::_createDB();
      }
      
      return $GLOBALS["FOF_DB_CONNECTION"];
   }

   /**
    * create a new database connection object
    * 
    * @param void
    * @return ADOConnection
    * @access private
    */
   function _createDB()
   {
      //instantiate DB connection object and connect to the database
      $db = ADONewConnection( FOF_DB_TYPE, 'date:pear:transaction' );


      if ( FOF_DB_DEBUG == true ) {
         $db->debug = true;
      }

      // connect to database using constants from config.php
      $retval = $db->Connect( FOF_DB_HOST, FOF_DB_USER, FOF_DB_PASS, FOF_DB_DBNAME );
      if (!$retval) {
         trigger_error("Error connecting to database, check settings in config.php..." ,E_USER_ERROR);
      }
      
      return $db;
   }

   function _int2bool( $intVal )
   {
      return $intVal == 1;
   }
   
   function _bool2int( $boolVal )
   {
      if ($boolVal == true) {
         return 1;
      }
      else {
         return 0;
      }   
   }
}


// define globals for singleton factory
$GLOBALS["FOF_MODEL_FACADE"]   = null;
$GLOBALS["FOF_FEED_MODEL"]     = null;
$GLOBALS["FOF_ITEM_MODEL"]     = null;
$GLOBALS["FOF_RSS_MODEL"]      = null;
$GLOBALS["FOF_OPTION_MODEL"]   = null;
$GLOBALS["FOF_USER_MODEL"]     = null;
$GLOBALS["FOF_TAG_MODEL"]      = null;
$GLOBALS["FOF_CATEGORY_MODEL"] = null;

/**
 * central place to fetch model instances
 * each model type is lazy loaded to reduce overhead
 * 
 */
class FOF_ModelFactory
{

   function getInstance()
   {
      if (is_null($GLOBALS['FOF_MODEL_FACADE'])) {
         require_once FOF_DIR . 'lib/model/facade.php';
         $GLOBALS["FOF_MODEL_FACADE"] = new FOF_ModelFacade();
      }
      return $GLOBALS["FOF_MODEL_FACADE"];
   }


   function getFeedModel()
   {
   	if (is_null($GLOBALS["FOF_FEED_MODEL"])) {
   		require_once FOF_DIR . 'lib/model/feed.php';
         $GLOBALS["FOF_FEED_MODEL"] = new FOF_FeedModel();
      }
      return $GLOBALS["FOF_FEED_MODEL"];
   }
   
   function getRssModel()
   {
   	if (is_null($GLOBALS["FOF_RSS_MODEL"])) {
         require_once FOF_DIR . 'lib/model/rss.php';
         $GLOBALS["FOF_RSS_MODEL"] = new FOF_RssModel();
      }
      return $GLOBALS["FOF_RSS_MODEL"];
   }
   
   function getOptionsModel()
   {
   	if (is_null($GLOBALS["FOF_OPTION_MODEL"])) {
         require_once 'option.php';
         $GLOBALS["FOF_OPTION_MODEL"] = new FOF_OptionsModel();
      }
      return $GLOBALS["FOF_OPTION_MODEL"];
   }
   
   function getItemModel()
   {
   	if (is_null($GLOBALS["FOF_ITEM_MODEL"])) {
         require_once 'item.php';
         $GLOBALS["FOF_ITEM_MODEL"] = new FOF_ItemModel();
      }
      return $GLOBALS["FOF_ITEM_MODEL"];
   }
   
   function getUserModel()
   {
      if (is_null($GLOBALS["FOF_USER_MODEL"])) {
         require_once 'user.php';
         $GLOBALS["FOF_USER_MODEL"] = new FOF_UserModel();
      }
      return $GLOBALS["FOF_USER_MODEL"];
   }
   
   function getTagModel()
   {
      if (is_null($GLOBALS["FOF_TAG_MODEL"])) {
         require_once 'tag.php';
         $GLOBALS["FOF_TAG_MODEL"] = new FOF_TagModel();
      }
      return $GLOBALS["FOF_TAG_MODEL"];
   }
   
   function getCategoryModel()
   {
   	if (is_null($GLOBALS["FOF_CATEGORY_MODEL"])) {
         require_once 'category.php';
         $GLOBALS["FOF_CATEGORY_MODEL"] = new FOF_CategoryModel();
      }
      return $GLOBALS["FOF_CATEGORY_MODEL"];
   }
}


/**
 * custom error handler for ADODB
 * 
 */
function fof_db_error_handler($dbms, $fn, $errno, $errmsg, $p1, $p2, &$thisConnection)
{
   if (error_reporting() == 0)
      return; // obey @ protocol

	$s = "<p class='error'>";
   switch($fn) {
      case 'EXECUTE':
         $sql = $p1;
         $inputparams = $p2;

         $s .= "<span>Error performing database operation, <a href=\"install.php\">Maybe you need to run the installer.</a></span>";
         $s .= "<br>$dbms error: [$errmsg] in $fn(\"$sql\")\n";
         break;

      case 'PCONNECT':
      case 'CONNECT':
         $host = $p1;
         $database = $p2;
         $s .= "<span>Unable to connect to the database, please verify config.php settings.</span>";
         $s .= "<br>$dbms error: [$errno: $errmsg] in $fn($host, '****', '****', $database)\n";
         break;

      default:
         $s .= "<span>Error performing database operation, <a href=\"install.php\">Maybe you need to run the installer.</a></span>";
         $s .= "<br>$dbms error: [$errno: $errmsg] in $fn($p1, $p2)\n";
         break;
   }
	$s .= "</p>";

   trigger_error($s, ADODB_ERROR_HANDLER_TYPE); 
}


