<?php

/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */

require_once 'base.php';


class FOF_FeedDDLHelper extends FOF_BaseDDLHelper
{
   function getCommands()
   {
   	$commands = array();
      $commands[] = new FOF_CreateFeedTable($this);
      $commands[] = new FOF_AddCategoryFeedColumn($this);
      $commands[] = new FOF_AddLatitudeFeedColumn($this);
      $commands[] = new FOF_AddLongitudeFeedColumn($this);
      $commands[] = new FOF_AddAutoTagColumn($this);
      $commands[] = new FOF_AddUpdateEnabledColumn($this);
      $commands[] = new FOF_AddLastUpdateColumnFeed($this);
      $commands[] = new FOF_AddUpdateIntervalColumn($this);
      $commands[] = new FOF_AddFeedCategoryIndex($this);
      
      return new FOF_ArrayIterator($commands);
   }
}


class FOF_CreateFeedTable extends FOF_Command
{
	var $commandName = "feed table create";
   
   var $table_def = "
   id I AUTO KEY,
   url C(250) NOTNULL DEFAULT '',
   title C(250) NOTNULL DEFAULT '',
   link X,
   category_id I DEFAULT 1,
   latitude N(14.8),
   longitude N(14.8),
   auto_tag I1 NOTNULL DEFAULT 0,
   update_enabled I1 NOTNULL DEFAULT 1,
   last_update TIMESTAMP
   ";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
      $tables = $this->helper->getTables();

   	if (!in_array(FOF_FEED_TABLE,$tables)) {
   
         $feedTableArray = $dd->CreateTableSQL(FOF_FEED_TABLE, $this->table_def);
   
         $val = $dd->ExecuteSQLArray($feedTableArray);
         if ($val) {
            $this->message = "table created";

            // refresh table list
            $this->helper->refreshTables();
         } else {
            $this->message = "create failed, " . $dd->connection->ErrorMsg();
            return false;
         }     
      } else {
         $this->message = "table exists";
      }
      
      return true;
   }
}


class FOF_AddCategoryFeedColumn extends FOF_Command
{
   var $commandName = "add column_id to feed";
   
   function execute()
   {
   	$dd = $this->helper->getDataDictionary();
   
   	// add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);
   
      // add category_id if missing
      if (!array_key_exists('CATEGORY_ID', $feed_columns)) {
   
         $catAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "category_id I DEFAULT 1");
   
         $retval = $dd->ExecuteSQLArray($catAddSql);
   
         if (!$retval) {
            $this->message = "add failed, " . $dd->connection->ErrorMsg();
            return false;
      
         }
         else {
            $this->message = "column added";
         }
      }
      else {
      	$this->message = "already updated";
      }
      return true;
   }  
}


class FOF_AddLatitudeFeedColumn extends FOF_Command
{
   var $commandName = "add latitude to feed";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);
   
      if (!array_key_exists('LATITUDE', $feed_columns) ) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "latitude N(14.8)");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "column added";
         }
      }
      else {
      	$this->message = "already updated";
      }
      return true;
   }
}

class FOF_AddLongitudeFeedColumn extends FOF_Command
{
   var $commandName = "add longitude to feed";

   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);

      if (!array_key_exists('LONGITUDE', $feed_columns)) {
   
         $lonAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "longitude N(14.8)");
   
         $retval = $dd->ExecuteSQLArray($lonAddSql);
   
         if (!$retval) {
            $this->message = "add failed, "  .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "column added";
         }         
      }
      else {
      	$this->message = "already updated";
      }
      return true;
      
   }
}

class FOF_AddAutoTagColumn extends FOF_Command
{
   var $commandName = "add auto_tag to feed";

   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);
      
      if (!array_key_exists('AUTO_TAG', $feed_columns) ) {
   
         $autotagAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "auto_tag I1 NOTNULL DEFAULT 0");
         $retval = $dd->ExecuteSQLArray($autotagAddSql);
      
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         }
         else {
            $this->message = "column added";
         }
      }
      else {
      	$this->message = "already updated";
      }
      return true;
   }
}

class FOF_AddUpdateEnabledColumn extends FOF_Command
{
   var $commandName = "add update_enabled to feed";

   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);

      if (!array_key_exists('UPDATE_ENABLED', $feed_columns)) {
   
         $catAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "update_enabled I1 NOTNULL DEFAULT 1");
   
         $retval = $dd->ExecuteSQLArray($catAddSql);
   
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "column added";
         }
      }
      else {
      	$this->message = "already updated";
      }
      return true;
   }
}


class FOF_AddLastUpdateColumnFeed extends FOF_Command
{
	var $commandName = "add last_update to feed";
   
   function execute()
   {
   	$dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);
      
         
      if (!array_key_exists('LAST_UPDATE', $feed_columns)) {
   
         $catAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "last_update TIMESTAMP");
   
         $retval = $dd->ExecuteSQLArray($catAddSql);
   
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "column added";

         }
      }
      else {
      	$this->message = "already updated";
      }
      return true;
   }
   
}

class FOF_AddUpdateIntervalColumn extends FOF_Command
{
   var $commandName = "add update_interval to feed";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $feed_columns = $dd->MetaColumns(FOF_FEED_TABLE);
      
         
      if (!array_key_exists('UPDATE_INTERVAL', $feed_columns)) {
   
         $catAddSql = $dd->AddColumnSQL(FOF_FEED_TABLE, "update_interval I");
   
         $retval = $dd->ExecuteSQLArray($catAddSql);
   
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "column added";
         }
      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}

class FOF_AddFeedCategoryIndex extends FOF_Command
{
   var $commandName = "add category_id index";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();

      $feed_indexes = $dd->MetaIndexes(FOF_FEED_TABLE);
   
      if (!array_key_exists(FOF_FEED_TABLE . '_category_id_idx', $feed_indexes)) {
         //index missing, create it
      
         //generate sql to add index
         $add_catid_ddl = $dd->CreateIndexSQL(FOF_FEED_TABLE . '_category_id_idx', FOF_FEED_TABLE, 'category_id');
   
         //execute the generated query
         $retval = $dd->connection->Execute($add_catid_ddl[0]);
      
            if (!$retval) {
               $this->message = "add failed, " .$dd->connection->ErrorMsg();
               return false;
            } else {
               $this->message = "index added";
            }
      }      
      else {
         $this->message = "already updated";
      }
      return true;

   }
}


