<?php

/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */



/**
 * @purpose provide install/uninstall helpers to the option model
 * @version $Revision: 296 $
 * $Id: base.php 466 2006-06-28 01:01:51Z kevbob $
 */

require_once 'base.php';


class FOF_ItemDDLHelper extends FOF_BaseDDLHelper
{
   function getCommands()
   {
   	$commands = array();
      $commands[] = new FOF_CreateItemTable($this);
      $commands[] = new FOF_AddFlagItemColumn($this);
      $commands[] = new FOFAddLatitudeItemColumn($this);
      $commands[] = new FOFAddLongitudeItemColumn($this);
      $commands[] = new FOFAddEnclosureLinkItemColumn($this);
      $commands[] = new FOFAddEnclosureTypeItemColumn($this);
      $commands[] = new FOFAddCacheTimeItemColumn($this);
      $commands[] = new FOFAddPublishTimeItemColumn($this);
      $commands[] = new FOF_AddItemFeedIndex($this);
      $commands[] = new FOF_AddItemFlagIndex($this);

      $commands[] = new FOFAddHashItemColumn($this);
      $commands[] = new FOF_AddItemHashIndex($this);
      
      return new FOF_ArrayIterator($commands);
   }
}


class FOF_CreateItemTable extends FOF_Command
{
	var $commandName = 'item table create';
   
   var $table_def = "
   id I AUTO KEY,
   feed_id I NOTNULL DEFAULT 0,
   cache_time TIMESTAMP NOTNULL,
   publish_time TIMESTAMP NOTNULL,
   link X,
   title C(250),
   content X,
   dccreator X,
   dcsubject X,
   flag I1,
   latitude N(14.8),
   longitude N(14.8),
   enclosure_link X,
   enclosure_type C(40),
   hash C(50)";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
      $tables = $this->helper->getTables();

      if (!in_array(FOF_ITEM_TABLE, $tables)) {
   
         $itemCreateSQL = $dd->CreateTableSQL(FOF_ITEM_TABLE, $this->table_def);
   
         $retval = $dd->ExecuteSQLArray($itemCreateSQL);
      
         if (!$retval) {
            $this->message = "create error: " . $dd->connection->ErrorMsg();
            return false;
         }
         else {
            $this->message = 'table created';
            // refresh table list
            
            $this->helper->refreshTables();

         }
      }
      else {
      	
         $this->message = 'table exists';
      }
      return true;      
   }
}


class FOF_AddFlagItemColumn extends FOF_Command
{
	var $commandName = 'add flag to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
   
      // if flag column is missing, add it and migrate any data from read column
      if (!array_key_exists('FLAG',$item_columns)) {
   
         $addFlagSQL = $fof_db_dictionary->AddColumnSQL(FOF_ITEM_TABLE, 'flag I1');
   
         $retval = $fof_db_dictionary->ExecuteSQLArray($addFlagSQL);
   
         if (!$retval) {
            $this->message = "add failed, " . $dd->connection->ErrorMsg();
            return false;
         }
         else {
            $this->message = 'column added';
         
         }
         // migrate data from read column if available
         if (in_array('READ', $item_columns)) {
      
         $dd->connection->Execute("UPDATE ".FOF_ITEM_TABLE.
                          "    SET flag = `read`" );
         }
      }
      else {
         $this->message = 'already updated';
      }
      return true;
      
   }
}


class FOFAddLatitudeItemColumn extends FOF_Command
{
   var $commandName = 'add latitude to item';
   
   function execute()
   {
   	$dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('LATITUDE',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "latitude N(14.8)");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
      }
      else {
      	$this->message = "already updated";
      }
      return true;
   }
   
}

class FOFAddLongitudeItemColumn extends FOF_Command
{
   var $commandName = 'add longitude to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('LONGITUDE',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "longitude N(14.8)");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}


class FOFAddEnclosureLinkItemColumn extends FOF_Command
{
   var $commandName = 'add enclosure link to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('ENCLOSURE_LINK',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "enclosure_link X");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}

class FOFAddEnclosureTypeItemColumn extends FOF_Command
{
   var $commandName = 'add enclosure type to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('ENCLOSURE_TYPE',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "enclosure_type C(40)");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}


class FOFAddCacheTimeItemColumn extends FOF_Command
{
   var $commandName = 'add cache_time to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('CACHE_TIME',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "cache_time TIMESTAMP NOTNULL");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
         
         //migrate values from timestamp column to cache_time column

         if (!array_key_exists('TIMESTAMP', $item_columns)) {
            // if old column name exists, migrate the data
            $dd->connection->Execute("UPDATE ".FOF_ITEM_TABLE." set cache_time = `timestamp`");
            $this->message .= ', data moved from timestamp column';
         }
         
      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}


class FOFAddPublishTimeItemColumn extends FOF_Command
{
   var $commandName = 'add publish_time to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('PUBLISH_TIME',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "publish_time TIMESTAMP NOTNULL");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
         
         if (array_key_exists('DCDATE', $item_columns)) {
            // migrate dcdate to publish_date
            $result = $dd->connection->Execute("select id, cache_time, dcdate from ".FOF_ITEM_TABLE);
      
            while ($row = $result->FetchRow() ) {
               //convert dcdate to unix epoch timestamp
               $unix_ts = strtotime($row['dcdate']);
         
               // if we couldn't parse dcdate, set the column to
               // the same value as cache_time
               if ( is_null($unix_ts)
                    or $unix_ts === -1
                    or $unix_ts === false) {
                    	

                  $unix_ts = $dd->connection->UnixTimeStamp($row['cache_time']);
               }
               $id = $row['id'];
               $dd->connection->Execute("update ".FOF_ITEM_TABLE.
                            "       SET publish_time = ".$fof_db->DBTimeStamp($unix_ts) .
                            " WHERE id = ?", array($id));
            }
            $result->Close();
            
            $this->message .= 'data moved from dcdate column';
         }
         
      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}

class FOFAddHashItemColumn extends FOF_Command
{
   var $commandName = 'add hash to item';
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
   
      // add/update columns on feed table
      $item_columns = $dd->MetaColumns(FOF_ITEM_TABLE);
      
      
      if (!array_key_exists('HASH',$item_columns)) {
   
         $latAddSql = $dd->AddColumnSQL(FOF_ITEM_TABLE, "hash C(50)");
   
         $retval = $dd->ExecuteSQLArray($latAddSql);
   
         if (!$retval) {
            $this->message = "add failed: " .$dd->connection->ErrorMsg();
            return false;

         } else {
            $this->message =  "column added";
         
         }
         
         // now populate values into hash column
         
         $result = $dd->connection->Execute("select id, link, title, content from ".FOF_ITEM_TABLE);
      
         while ($row = $result->FetchRow() ) {

               /*
                  create hash value for row
                  this logic copied from RSS_Item::getUniqueHash()
                  they both must remain identical      */
               $inputString  = $row['link'];
               $inputString .= $row['title'];
               $inputString .= $row['content'];
               $itemHash = sha1($inputString);

               $dd->connection->Execute("update ".FOF_ITEM_TABLE.
                            "       SET hash = ?" .
                            " WHERE id = ?",
                            array($itemHash,(int)$row['id']));
            }
            $result->Close();

      }
      else {
         $this->message = "already updated";
      }
      return true;
   }
}


class FOF_AddItemHashIndex extends FOF_Command
{
   var $commandName = "add hash index";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();

      $item_indexes = $dd->MetaIndexes(FOF_ITEM_TABLE);
   
      if (!array_key_exists(FOF_ITEM_TABLE.'_hash_idx', $item_indexes)) {
         //index missing, create it
      
         //generate sql to add index
         $add_feed_idx_ddl = $dd->CreateIndexSQL(FOF_ITEM_TABLE.'_hash_idx', FOF_ITEM_TABLE, 'hash');
   
         //execute the generated query
         $retval = $dd->connection->Execute($add_feed_idx_ddl[0]);
      
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "index added";
         }
      }      
      else {
         $this->message = "already updated";
      }
      return true;

   }
}




class FOF_AddItemFeedIndex extends FOF_Command
{
   var $commandName = "add feed_id index";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();

      $item_indexes = $dd->MetaIndexes(FOF_ITEM_TABLE);
   
      if (!array_key_exists(FOF_ITEM_TABLE.'_feed_id_idx', $item_indexes)) {
         //index missing, create it
      
         //generate sql to add index
         $add_feed_idx_ddl = $dd->CreateIndexSQL(FOF_ITEM_TABLE.'_feed_id_idx', FOF_ITEM_TABLE, 'feed_id');
   
         //execute the generated query
         $retval = $dd->connection->Execute($add_feed_idx_ddl[0]);
      
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "index added";
         }
      }      
      else {
         $this->message = "already updated";
      }
      return true;

   }
}


class FOF_AddItemFlagIndex extends FOF_Command
{
   var $commandName = "add flag index";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();

      $item_indexes = $dd->MetaIndexes(FOF_ITEM_TABLE);
   
      if (!array_key_exists(FOF_ITEM_TABLE . '_flag_idx', $item_indexes)) {
         //index missing, create it
      
         //generate sql to add index
         $add_flag_idx_ddl = $dd->CreateIndexSQL(FOF_ITEM_TABLE . '_flag_idx', FOF_ITEM_TABLE, 'flag');
   
         //execute the generated query
         $retval = $dd->connection->Execute($add_flag_idx_ddl[0]);
      
         if (!$retval) {
            $this->message = "add failed, " .$dd->connection->ErrorMsg();
            return false;
         } else {
            $this->message = "index added";
         }
      }      
      else {
         $this->message = "already updated";
      }
      return true;

   }
}
