<?php

/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose provide install/uninstall helpers to the option model
 * @version $Revision: 296 $
 * $Id: base.php 466 2006-06-28 01:01:51Z kevbob $
 */

require_once 'base.php';


class FOF_OptionDDLHelper extends FOF_BaseDDLHelper
{
	
   function getCommands()
   {
   	$commands = array();
      $commands[] = new FOF_CreateOptionTable($this);
      $commands[] = new FOF_LoadDefaultOptions($this);
      
      return new FOF_ArrayIterator($commands);
   }
   
}

class FOF_CreateOptionTable extends FOF_Command
{
	var $commandName = 'option table create';

   var $table_def = "
   name C(250) NOTNULL KEY,
   value X";

   function execute()
   {
   	$dd = $this->helper->getDataDictionary();
      $tables = $this->helper->getTables();
      
      if (!in_array(FOF_CONFIG_TABLE, $tables)) {
      
         $confTableSQL = $dd->CreateTableSQL(FOF_CONFIG_TABLE,$this->table_def);
      
         $val = $dd->ExecuteSQLArray($confTableSQL);
      
         if (!$val) {
         	$this->message = 'create failed: ' . $dd->connection->ErrorMsg();
            return false;

         }
         else {
            $this->message = 'table created';

            // refresh table list
            $this->helper->refreshTables();
      }
   } else {
      // this shouldn't happen but print just in case
      $this->message = 'table exists';
   }
   
      return true;
   }

}

class FOF_LoadDefaultOptions extends FOF_Command
{
   var $commandName = 'load default options';

   var $defaultFile = 'default.config';
   
   function execute()
   {
      require_once FOF_DIR . $this->defaultFile;
      $opt_model = $this->helper->model;
      $opts = $opt_model->fetch();
   
      $addList = array();
      foreach ($fof_default_options as $key => $value) {
         if (is_null($opts->get($key))){
            $opts->set($key, $value);
            $addList[] = $key;
         }
      }
      if (count($addList) > 0) {
      	$this->message = 'added: ' . join(', ', $addList);
      }
      else {
      	$this->message = 'all options present';
      }
   
      return true;	

   }
}


