<?php

/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose provide install/uninstall helpers to the tag model
 * @version $Revision: 296 $
 * $Id: base.php 466 2006-06-28 01:01:51Z kevbob $
 */

require_once 'base.php';



class FOF_TagDDLHelper extends FOF_BaseDDLHelper
{
   function getCommands()
   {
      $commands = array();
      $commands[] = new FOF_CreateTagTable($this);
      $commands[] = new FOF_CreateTagXItemTable($this);
      $commands[] = new FOF_CreateTagAliasTable($this);
      $commands[] = new FOF_FixAllTimestamps($this);
      
      return new FOF_ArrayIterator($commands);
   }
   
}


class FOF_CreateTagTable extends FOF_Command
{
   var $commandName = 'tag table create';
   
   var $table_def = "
   id I AUTO KEY,
   name C(250) NOTNULL
";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
      $tables = $this->helper->getTables();

      if (!in_array(FOF_TAG_TABLE, $tables)) {
   
         $itemCreateSQL = $dd->CreateTableSQL(FOF_TAG_TABLE, $this->table_def);
   
         $retval = $dd->ExecuteSQLArray($itemCreateSQL);
      
         if (!$retval) {
            $this->message = "create error: " . $dd->connection->ErrorMsg();
            return false;
         }
         else {
            $this->message = 'table created';
            // refresh table list
            
            $this->helper->refreshTables();

         }
      }
      else {
         
         $this->message = 'table exists';
      }
      return true;      
   }
}


class FOF_CreateTagXItemTable extends FOF_Command
{
   var $commandName = 'tag_x_item table create';
   
   var $table_def = "
   tag_id I KEY,
   item_id I KEY
";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
      $tables = $this->helper->getTables();

      if (!in_array(FOF_X_TAG_ITEM_TABLE, $tables)) {
   
         $itemCreateSQL = $dd->CreateTableSQL(FOF_X_TAG_ITEM_TABLE, $this->table_def);
   
         $retval = $dd->ExecuteSQLArray($itemCreateSQL);
      
         if (!$retval) {
            $this->message = "create error: " . $dd->connection->ErrorMsg();
            return false;
         }
         else {
            $this->message = 'table created';
            // refresh table list
            
            $this->helper->refreshTables();

         }
      }
      else {
         
         $this->message = 'table exists';
      }
      return true;      
   }
}


class FOF_CreateTagAliasTable extends FOF_Command
{
   var $commandName = 'tag alias table create';
   
   var $table_def = "
   name C(250) KEY,
   tag_id I NOTNULL

";
   
   function execute()
   {
      $dd = $this->helper->getDataDictionary();
      $tables = $this->helper->getTables();

      if (!in_array(FOF_TAG_ALIAS_TABLE, $tables)) {
   
         $itemCreateSQL = $dd->CreateTableSQL(FOF_TAG_ALIAS_TABLE, $this->table_def);
   
         $retval = $dd->ExecuteSQLArray($itemCreateSQL);
      
         if (!$retval) {
            $this->message = "create error: " . $dd->connection->ErrorMsg();
            return false;
         }
         else {
            $this->message = 'table created';
            // refresh table list
            
            $this->helper->refreshTables();

         }
      }
      else {
         
         $this->message = 'table exists';
      }
      return true;      
   }
}

class FOF_FixAllTimestamps extends FOF_Command
{
	var $commandName = 'fix mysql timestamp columns';
   
   function execute()
   {
   	
      // change default for timestamp columns on mysql tables.  This correct bug #1477023 
      if (FOF_DB_TYPE == 'mysql') {
         $dd = $this->helper->getDataDictionary();
         // construct reference to all columns that are timestamps.
         $fof_timestamps = array( array(FOF_FEED_TABLE, 'last_update'),
                                  array(FOF_ITEM_TABLE, 'cache_time'),
                                  array(FOF_ITEM_TABLE, 'publish_time'),
                                  array(FOF_USERS_TABLE, 'date_added'),
                                  array(FOF_USERS_TABLE, 'date_changed'));

         foreach ($fof_timestamps as $columnInfo) {
            // get info for tables columns
            $tName = $columnInfo[0];
            $cName = $columnInfo[1];
            $colMetaData = $dd->MetaColumns($columnInfo[0]);
            $cNameUpper = strtoupper($cName);
            if (isset($colMetaData[$cNameUpper])) {
               if ($colMetaData[$cNameUpper]->default_value == 'CURRENT_TIMESTAMP') {
                  $dd->connection->Execute("ALTER TABLE $tName CHANGE $cName $cName TIMESTAMP NOT NULL DEFAULT 0");
               }
            }
         }
         $this->message = 'done.';
      }
      return true;
   }
}



