<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose represent an item within the system
 * @version $Revision: 296 $
 * $Id: item.php 520 2006-09-07 08:10:35Z kevbob $
 */

require_once 'item_search.php';
require_once 'feed.php';
require_once 'category.php';
require_once 'tag.php';
require_once 'iterator.php';


#define accepted values for item flag column
define('FOF_READ_FLAG', 1);
define('FOF_UNREAD_FLAG', 0);
define('FOF_SAVED_FLAG', 2);


class FOF_ItemModel extends FOF_BaseModel
{
   
   /**
    * return new instance of item
    * 
    * @param void
    * @return FOF_Item
    */
   function create()
   {
      return new FOF_Item();
   }
   
   function install()
   {
   	require_once 'ddl/item.php';
      $ddl_helper = new FOF_ItemDDLHelper($this);
      return $ddl_helper->getCommands();
      
   }
   
   /**
    * retrieve an item from db using primary key
    * 
    * @param int $id
    * @return FOF_Item
    */
   function fetch( $id )
   {
      $db = $this->getDB();
      $query = "SELECT i.id, i.flag as item_read, f.title as feed_title," .
            "       f.link as feed_link," .
            "       f.id as feed_id," .
	         "       f.latitude as feed_latitude, f.longitude as feed_longitude," .
            "       i.link as item_link, i.title as item_title," .
            "       i.flag," .
            "       i.cache_time," .
            "       i.content," .
            "       i.publish_time," .
            "       i.dccreator," .
            "       i.latitude, i.longitude," .
            "       cat.id as category_id, cat.title as category_title," .
            "       i.enclosure_link as enclosure_link," .
            "       i.enclosure_type as enclosure_type," .
            "       f.url as feed_url," .
            "       i.hash" .
            "  FROM ".FOF_FEED_TABLE." f" .
            "       JOIN ".FOF_CATEGORY_TABLE." cat ON (f.category_id = cat.id)" .
            "       JOIN ".FOF_ITEM_TABLE." i ON (i.feed_id = f.id)" .
            " WHERE i.id = ?";
      $row = $db->getRow($query, array((int)$id));

      return $this->row2instance($row);
      
   }
   
   /**
    * search for item using unique hash value
    * 
    * @param string $hashString
    * @return FOF_Item
    */
   function findItem( $hashString )
   {
      $db = $this->getDB();

      $query = "SELECT i.id, i.flag as item_read, f.title as feed_title," .
            "       f.link as feed_link," .
            "       f.id as feed_id," .
            "       i.link as item_link, i.title as item_title," .
            "       f.latitude as feed_latitude, f.longitude as feed_longitude," .
            "       i.flag," .
            "       i.cache_time," .
            "       i.content," .
            "       i.publish_time," .
            "       i.dccreator," .
            "       i.latitude, i.longitude," .
            "       cat.id as category_id, cat.title as category_title," .
            "       i.enclosure_link as enclosure_link," .
            "       i.enclosure_type as enclosure_type," .
            "       f.url as feed_url," .
            "       i.hash" .
            "  FROM ".FOF_FEED_TABLE." f" .
            "       JOIN ".FOF_CATEGORY_TABLE." cat ON (f.category_id = cat.id)" .
            "       JOIN ".FOF_ITEM_TABLE." i ON (i.feed_id = f.id)" .
            " WHERE i.hash = ?";
      $row = $db->getRow($query, array($hashString));

      return $this->row2instance($row);
   }



   /**
    * main search method for items
    * 
    * @param FOF_ItemSearch $search
    * @param bool $returnExistence return true/false if search matched any results
    * @return FOF_Iterator
    */
   function search( $search, $returnExistence = false )
   {
      $db = $this->getDB();
      
      $query = "SELECT i.id, i.flag as item_read, f.title as feed_title,\n" .
            "       f.link as feed_link,\n" .
            "       f.id as feed_id,\n" .
	         "       f.latitude as feed_latitude, f.longitude as feed_longitude,\n" .
            "       i.link as item_link, i.title as item_title,\n" .
            "       i.flag,\n" .
            "       i.cache_time,\n" .
            "       i.content,\n" .
            "       i.publish_time,\n" .
            "       i.dccreator,\n" .
            "       i.latitude, i.longitude,\n" .
            "       cat.id as category_id, cat.title as category_title,\n" .
            "       i.enclosure_link as enclosure_link," .
            "       i.enclosure_type as enclosure_type," .
            "       f.url as feed_url,\n" .
            "       i.hash" .
            "  FROM ".FOF_FEED_TABLE." f\n" .
            "       JOIN ".FOF_CATEGORY_TABLE." cat ON (f.category_id = cat.id)\n" .
            "       JOIN ".FOF_ITEM_TABLE." i ON (i.feed_id = f.id)\n";

      if ( $returnExistence ) {
      	
         $query = "SELECT 1" .
                  "  FROM ".FOF_FEED_TABLE." f\n" .
                  "       JOIN ".FOF_CATEGORY_TABLE." cat ON (f.category_id = cat.id)\n" .
                  "       JOIN ".FOF_ITEM_TABLE." i ON (i.feed_id = f.id)\n";
      }

      $filter = $search->getFilter();

      // only if there is a filter on a tag, do we join in the tag join table.
      if ($filter->findFilter('tag_id')) {
         $query .= " JOIN ".FOF_X_TAG_ITEM_TABLE." ti ON (i.id = ti.item_id)\n";
      }

      list($whereStuff, $queryArgs) = $filter->getSQL();
      $query .= " WHERE " . $whereStuff;

      // do not perform sort if we are testing for return existence
      if ( !$returnExistence ) {
      
         $query .= sprintf(" ORDER BY %s %s, i.id %s",
                           $search->getSortField(),
                           $search->getSortOrder(),
                           $search->getSortOrder() );
      }
      
      if ($returnExistence) {
         $result = $db->SelectLimit($query,
                                    1,
                                    $search->getOffset(),
                                    $queryArgs);
      }
      // only use search limit query if we have a limit or offset specified
      elseif ( $search->getLimit() != 0 ) {
         $result = $db->SelectLimit($query,
                                    $search->getLimit(),
                                    $search->getOffset(),
                                    $queryArgs);
      }
      else {
         $result = $db->Execute($query,
                                $queryArgs);
      }
      
      if ($returnExistence) {
      	if ($result != false
             and $result->RecordCount() > 0) {
            return true;
         }
         else {
         	return false;
         } 
      }
      if ($result != false) {
         $result_it = new FOF_ModelIterator($result, $this);
      }
      else {
         $result_it = new FOF_EmptyIterator();   
      }
      return $result_it;
       
   }
   
   
   /**
    * get a search instance pre-populated with default values
    * TODO possible use users's options here also
    * 
    * @param void
    * @return FOF_ItemSearch
    */
   function getDefaultSearch()
   {
      $search = new FOF_ItemSearch();
      $search->setFlag(FOF_UNREAD_FLAG);
      $search->setSortField('publish_time');
      $search->setSortOrder('DESC');
      return $search;
   }

   
   /**
    * add tag to item
    * 
    * @param FOF_Item $item
    * @param FOF_Tag  $tag
    * @return boolean
    */
   function addTag( $item, $tag )
   {
      $db = $this->getDB();
      
      // check for duplicate
      $exists = $db->GetOne("SELECT tag_id" .
                            "  FROM ".FOF_X_TAG_ITEM_TABLE .
                            " WHERE item_id = ?" .
                            "   AND tag_id = ?",
                            array((int)$item->getId(), (int)$tag->getId()));
      if ($exists) {
         return false;
      }
   
      // relation does not exist, add it
      $retval = $db->Execute("INSERT INTO ".FOF_X_TAG_ITEM_TABLE." (item_id, tag_id)" .
                             "            VALUES( ?, ?)",
                             array((int)$item->getId(), (int)$tag->getId()));
      
      return $retval;
   }
   
   /**
    * remove tag from item
    * 
    * @param FOF_Item $item
    * @param FOF_Tag  $tag
    * @return boolean
    */
   function removeTag( $item, $tag )
   {
      $db = $this->getDB();
      $retval = $db->Execute("DELETE FROM ".FOF_X_TAG_ITEM_TABLE .
                             " WHERE item_id = ?" .
                             "   AND tag_id = ?",
                             array((int)$item->getId(), (int)$tag->getId()));
      return $retval;
   }

   
   /**
    * remove item from database
    * 
    * @param FOF_Item $item
    * @return boolean
    */
   function remove( &$item )
   {
      $db = $this->getDB();
      
      // remove any tags on item
      $retval = $db->Execute("DELETE FROM ".FOF_X_TAG_ITEM_TABLE.
                             " WHERE item_id = ?", array((int)$item->getId()));
      
      // if it fails, return
      if (!$retval) {
         return false;
      }
      
      $retval = $db->Execute("DELETE FROM ".FOF_ITEM_TABLE.
                             " WHERE id = ?", array((int)$item->getId()));
      $item->setId(-1);
      return $retval;
   }
   
   /**
    * add/update item in the database
    * 
    * @param FOF_Item $item
    * @return boolean
    */
   function save( &$item )
   {
      $db = $this->getDB();
      
      if ( $item->getId() == -1) {
         // item should be inserted
         $feed = $item->getFeed();
         
         $sql = "insert into ". FOF_ITEM_TABLE .
                "            (feed_id,link,title,content,cache_time," .
                "             publish_time,dccreator,latitude," .
                "             longitude,enclosure_link,enclosure_type,flag," .
                "             hash)" .
                "     values (?, ?, ?, ?, ".$db->sysTimeStamp."," .
                "             ".$db->DBTimeStamp($item->getPublishTime()).", ?, ?, ?, ?, ?, ?, ?)";

         $retval = $db->Execute($sql, array((int)$feed->getId(),
                                            $item->getLink(),
                                            $item->getTitle(),
                                            $item->getContent(),
                                            $item->getCreator(),
                                            $item->getLatitude(),
                                            $item->getLongitude(),
                                            $item->getEnclosureLink(),
                                            $item->getEnclosureType(),
                                            $item->getFlag(),
                                            $item->getHash()));


         $new_item_id = $db->Insert_ID();

         // postgres's Insert_ID() method is broken. 
         // this is a hack to get around it but, the name
         // of the sequence may be predictable like below
         if (!$new_item_id) {
            $seqName = FOF_ITEM_TABLE . '_id_seq';
            $new_item_id = $db->GetOne("select currval('$seqName')");
         }
         $item->setId( $new_item_id );
         
      }
      else {
         // item should be updated
         
         $sql = "UPDATE ".FOF_ITEM_TABLE .
                "       SET title = ?," .
                "           content = ?," .
                "           flag = ?," .
                "           latitude = ?," .
                "           longitude = ?," .
                "           enclosure_link = ?," .
                "           enclosure_type = ?," .
                "           publish_time = ". $db->DBTimeStamp($item->getPublishTime()) .", ".
                "           cache_time = " .  $db->DBTimeStamp($item->getCacheTime()).", " .
                "           hash = ? " .
                " WHERE id = ?";
         $retval = $db->Execute($sql, array($item->getTitle(),
                                            $item->getContent(),
                                            $item->getFlag(),
                                            $item->getLatitude(),
                                            $item->getLongitude(),
                                            $item->getEnclosureLink(),
                                            $item->getEnclosureType(),
                                            $item->getHash(),
                                            (int)$item->getId()));
      }
      
   }
   
   /**
    * convert a db result row to an FOF_Item instance
    * 
    * @param array $row
    * @param boolean $extended (should use extended class)
    */
   function row2instance( $row, $extended = true )
   {
      if ($row == false) {
         return false;
      }
      
      $db = $this->getDB();
      
      if ( $extended ) {
         $item = new FOF_ItemEx( $row['id'], $this );
      }
      else {
         $item = new FOF_Item( $row['id']);
      }

      $feed = new FOF_FeedExt($row['feed_id'],FOF_ModelFactory::getFeedModel());
      $feed->setTitle($row['feed_title']);
      $feed->setLink($row['feed_link']);
      $feed->setUrl($row['feed_url']);
      $feed->setLatitude($row['feed_latitude']);
      $feed->setLongitude($row['feed_longitude']);
      
      $cat = new FOF_Category($row['category_id']);
      $cat->setTitle($row['category_title']);
      $feed->setCategory($cat);
      $item->setFeed($feed);

      $item->setLink($row['item_link']);
      $item->setTitle($row['item_title']);
      $item->setCacheTime($db->UnixTimeStamp($row['cache_time']));
      $item->setPublishTime($db->UnixTimeStamp($row['publish_time']));
      $item->setContent($row['content']);
      $item->setFlag($row['flag']);
      $item->setCreator($row['dccreator']);
      $item->setLatitude($row['latitude']);
      $item->setLongitude($row['longitude']);
      $item->setEnclosureLink($row['enclosure_link']);
      $item->setEnclosureType($row['enclosure_type']);
      $item->setHash($row['hash']);
      return $item;
   }
}


/**
 * data transfer class for items
 *
 */
class FOF_Item
{
   var $id;
   var $feed;
   var $cache_time;
   var $publish_time;
   var $link;
   var $title;
   var $content;
   var $dccreator;
   var $flag;
   var $latitude;
   var $longitude;
   var $enclosure_link;
   var $enclosure_type;
   var $hash;

   function FOF_Item( $id = -1 )
   {
      $this->id = $id;
   }
   
   function getId()
   {
      return $this->id;
   }
   
   function setId( $id )
   {
      $this->id = $id;
   }

   function getFeed()
   {
      return $this->feed;
   }
   
   function setFeed( $feed )
   {
      $this->feed = $feed;
   }
   
   function getCacheTime()
   {
      return $this->cache_time;
   }
   
   function setCacheTime( $cache_ts )
   {
      $this->cache_time = $cache_ts;
   }
   
   function getPublishTime()
   {
      return $this->publish_time;
   }
   
   function setPublishTime( $publish_ts )
   {
      $this->publish_time = $publish_ts;
   }
   
   function getLink()
   {
      return $this->link;
   }
   
   function setLink( $link )
   {
      $this->link = $link;
   }
   
   function getTitle()
   {
      return $this->title;  
   }
   
   function setTitle( $title )
   {
      $this->title = $title;
   }
   
   function getContent()
   {
      return $this->content;
   }
   
   function setContent( $content )
   {
      $this->content = $content;
   }
   
   function getCreator()
   {
      return $this->dccreator;
   }
   
   function setCreator( $creator )
   {
      $this->dccreator = $creator;
   }
   
   function getFlag()
   {
      return $this->flag;
   }
   
   function setFlag( $flag )
   {
      $this->flag = $flag;
   }
   
   function getLatitude()
   {
      return $this->latitude;
   }
   
   function setLatitude( $lat )
   {
      $this->latitude = $lat;
   }
   
   function getLongitude()
   {
      return $this->longitude;
   }
   
   function setLongitude( $long )
   {
      $this->longitude = $long;
   }
   
   function getEnclosureLink()
   {
      return $this->enclosure_link;
   }
   
   function setEnclosureLink( $link )
   {
      $this->enclosure_link = $link;
   }
   
   function getEnclosureType()
   {
      return $this->enclosure_type;
   }
   
   function setEnclosureType( $type )
   {
      $this->enclosure_type = $type;
   }
   
   function getHash()
   {
   	return $this->hash;
   }
   
   function setHash( $hash )
   {
   	$this->hash = $hash;
   }
}


class FOF_ItemEx extends FOF_Item
{
   var $model;
   
   function FOF_ItemEx( $id, $model )
   {
      parent::FOF_Item( $id );
      $this->model = $model;
   }

   /**
    * get tags attached to this item
    */
   function getTags()
   {
      $tag_model = FOF_ModelFactory::getTagModel();
      return $tag_model->search($this);
   }

   function addTag( $tag )
   {
      $this->model->addTag($this, $tag);
   }
   
   function removeTag( $tag )
   {
      $this->model->removeTag($this, $tag);
   }
}

