<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose class for the options of the specific user
 * @version $Revision: 432 $
 * $Id: option.php 493 2006-08-09 13:08:12Z kevbob $
 */

// load default options
//require_once FOF_DIR . 'default.config';


class FOF_OptionsModel extends FOF_BaseModel
{

   function install()
   {
      require_once 'ddl/option.php';
   	$ddl_helper = new FOF_OptionDDLHelper($this);
      return $ddl_helper->getCommands();
   }

   /**
    * retrieve an options instance
    * 
    * @param void
    * @return FOF_Options
    */
   function fetch()
   {
      // create FOF_Option instance
      $opt = new FOF_Options($this);
      
   	return $opt;
   }

   function insertValue($key, $value)
   {
      $db = $this->getDB();

      //add new entry
      $db->Execute("INSERT INTO ".FOF_CONFIG_TABLE.
                   " (name, value) " .
                   " VALUES(?,?)", array($key, $value));
      
   }
   
   function updateValue($key, $value)
   {
      $db = $this->getDB();

      //update entry
      $db->Execute("UPDATE ".FOF_CONFIG_TABLE.
                   " SET value = ? " .
                   " WHERE name = ?", array($value, $key));
   }

   function deleteValue($key)
   {
      $db = $this->getDB();

      //remove entry
      $db->Execute("DELETE FROM ".FOF_CONFIG_TABLE.
                   " WHERE name = ?", array($key));
   }

   /**
    * retrieve all name/value pairs from database
    *
    * @param void
    * @return array
    */
   function dump()
   {
      $db = $this->getDB();
      $options = array();
      
		$result = $db->Execute( "SELECT name, value" .
                              "  FROM ".FOF_CONFIG_TABLE );
      
      if ($result == false) {
      	return false;
      }
      
		while ($row = $result->FetchRow()) {
			$options[$row['name']] = $row['value'];
		}
		$result->Close();
      
      return $options;
   }
}

class FOF_Options
{

   /**
     *  where to store cached values pulled from db
     * @var array $vals
     */
   var $vals;
   
   /**
    * store options model so we can perform CRUD operations
    * @var FOF_OPtionsModel $model
    */
   var $model;
   
   /**
    * @param FOF_OptionsModel $model
    */
   function FOF_Options($model)
   {
      $this->model = $model;
      $this->vals =& $model->dump();
   }

   /**
    * retrieve option value from key
    * returns null if key does not exist
    * 
    * @param string
    * @return string or null
    */
   function get($key)
   {
      if (array_key_exists($key, $this->vals)) {
         return $this->vals[$key];
      }
      else {
         return null;
      }
   }
   
    /**
     * save a parameter to the database
     *
     * @param string $name   name of the option to change
     * @param string $value  value of the option to store
     * @return void
     */
   function set($key, $value)
    {
      if (array_key_exists($key, $this->vals)) {
         $this->model->updateValue($key, $value);
      }
      else {
         $this->model->insertValue($key, $value);
      }
      
      // locally cache new value
      $this->vals[$key] = $value;
   }
   
   /**
    * remove and option key
    * 
    * @param string
    * @return void
    */
   function remove($key)
   {
      $this->model->deleteValue($key);
      unset($this->vals[$key]);
   }
}

