<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose represent a tag within the system
 * @version $Revision: 296 $
 * $Id: tag.php 493 2006-08-09 13:08:12Z kevbob $
 */

require_once 'iterator.php';


class FOF_TagModel extends FOF_BaseModel
{
   /**
    * get new, empty tag instance
    * 
    * @param void
    * @return FOF_Tag
    */
   function create()
   {
      return new FOF_Tag();
   }
   
   function install()
   {
   	
      require_once 'ddl/tag.php';
   	$ddl_helper = new FOF_TagDDLHelper($this);
      return $ddl_helper->getCommands();
      
   }
   
   /**
    * get tag from db using primary key (id)
    * 
    * @param int $id
    * @return FOF_Tag
    */
   function fetch( $id )
   {
      $db = $this->getDB();
      
      $row = $db->GetRow("SELECT id, name" .
                          "  FROM ".FOF_TAG_TABLE.
                          " WHERE id = ?", array((int)$id));
      return $this->row2instance($row);
   }
   
   /**
    * get tags for an item
    */
   function search( $item )
   {
      $db = $this->getDB();
      
      $result = $db->Execute("SELECT t.id, t.name" .
                          "  FROM ".FOF_TAG_TABLE." t ".
                          " JOIN ".FOF_X_TAG_ITEM_TABLE. " tx ON (t.id = tx.tag_id)".
                          " WHERE tx.item_id = ?".
                          " ORDER BY name", array((int)$item->getId()));

      return new FOF_ModelIterator($result, $this);
   }
   
   /**
    * store tag in persistent storage (db)
    * 
    * @param FOF_Tag $tag
    * @return boolean
    */
   function save( &$tag )
   {
      $db = $this->getDB();
      
      if ( $tag->getId() == -1 ) {
         // insert tag
         $retval = $db->Execute("INSERT INTO ".FOF_TAG_TABLE.
                                "  (name)" .
                                "  VALUES(?)",
                                  array($tag->getName()));
         
         $new_id = $db->Insert_ID();
         
         // postgres's Insert_ID() method is broken. 
         // this is a hack to get around it but, the name
         // of the sequence may be predictable like below
         if (!$new_id) {
            $seqName = FOF_TAG_TABLE . '_id_seq';
            $new_id = $db->GetOne("select currval('$seqName')");
         }
         $tag->setId( $new_id );
         
      }
      else {
         // update tag
         
         $retval = $db->Execute("UPDATE ".FOF_TAG_TABLE.
                                "       SET name = ?" .
                                " WHERE id = ?", array($tag->getName(),
                                                       (int)$tag->getId()));
      }
      return $retval;
   }

   /**
    * remove tag
    * disassociate all items from tag
    * 
    * @param FOF_Item $tag
    * @return void
    */
   function remove( &$item )
   {
      $db = $this->getDB();
      
      // remove all items from tag
      $db->Execute( "DELETE FROM ".FOF_X_TAG_ITEM_TABLE.
                    " WHERE tag_id = ?", array((int)$item->getId()) );

      $db->Execute( "DELETE FROM ".FOF_TAG_TABLE.
                    " WHERE id = ?", array((int)$item->getId()) );
      
      // disassociate item object from db from db
      $item->setId(-1);
   }

   /**
    * retrieve iterator of all tags in the database
    * 
    * @param void
    * @return FOF_Iterator
    */
   function getTags()
   {
      $db = $this->getDB();
      
      $result = $db->Execute("SELECT id, name" .
                          "  FROM ".FOF_TAG_TABLE .
                          " ORDER BY name");
      if ($result != false) {
         $result_it = new FOF_ModelIterator($result, $this);
      }
      else {
         $result_it = new FOF_EmptyIterator();   
      }
      return $result_it;
      
   }
   
   /**
    * 
    * @param array $row
    * @return FOF_Tag or false
    */
   function row2instance( $row )
   {
      if ($row == false) {
         return false;
      }
      
      $tag = new FOF_Tag( $row['id'] );
      $tag->setName( $row['name']);
      
      // populate aliases for tag
      
      return $tag;
   }
}


/**
 * object container to hold tag
 *
 */
class FOF_Tag
{
   var $id;
   var $name;
   var $aliases;

   function FOF_Tag( $id = -1 )
   {
      $this->id = $id;
      $this->aliases = array();
   }
   
   // declare getter and setter methods for fields
   
   function getId()
   {
      return $this->id;
   }
   
   function setId( $id )
   {
      $this->id = $id;
   }
   
   function getName()
   {
      return $this->name;
   }
   
   function setName( $name )
   {
      $this->name = $name;
   }

// TODO implement tag aliases
//   function addAlias( $alias )
//   {
//   }
//
//   function removeAlias( $alias )
//   {
//   }
//   
//   function getAliases()
//   {
//      return array();
//   }
}

