<?php
/**
 * AJAX extension to the PHP-XMLRPC lib (works with jsonrpc, too).
 *
 * Makes use of the excellent Jsolait javascript library by Jan-Klaas Kollhof
 *
 * Original idea taken from the PHP-O-Lait library by Craig Mason-Jones
 *
 * @author Gaetano Giunta
 * @version $Id: ajaxmlrpc.inc,v 1.3 2006/02/05 23:05:53 ggiunta Exp $
 * @copyright (c) 2006 G. Giunta
 *
 * @todo add a js object wrapper for all web services if user prefers oop instead
 *       of plain function names (see php-o-lait for an example),
 *       or at least a name prefix for all created functions, to prevent js namespace pollution
 * @todo find a fix for xmlrpc methods whose name contain chars invalid in js function names (eg. '.')
 */

	// requires: xmlrpc.inc, xmlrpcs.inc

	class js_rpc_server extends xmlrpc_server
	{
		var $jsolaitPath = 'jsolait'; // default url of jsolait lib: same dir as php script...
		var $selfUrl;

		/**
		* Override base class creator, since we take care of auto-executing
		* (this means on creating class we either execute-and-die or basically
		* do nothing)
		*/
		function js_rpc_server($dispMap=null)
		{
			// Check if this page has been called via XMLRPC or not
			if ($_SERVER['REQUEST_METHOD'] == 'POST' && isset($_SERVER['CONTENT_TYPE']) &&
			strpos(strtolower($_SERVER['CONTENT_TYPE']), 'text/xml') === 0)
			{
				// xmlrpc call received: serve web service and die
				parent::xmlrpc_server($dispMap, true);
				die();
			}
			else
			{
				// standard page request received: prepare self to emit javascript
				// code that will call self
				$this->selfUrl = $_SERVER['REQUEST_URI'];
				parent::xmlrpc_server($dispMap, false);
			}
		}

		/**
		* Echo js code that wraps calls to server.
		* Note that method names with a . will not work!
		* @param array $methodlist list of methods to be wrapped. Null = all server methods (except system.XXX methods)
		*/
		function importMethods2JS($methodlist=null)
		{
			echo js_wrap_xmlrpc_server($this, $this->selfUrl, $this->jsolaitPath, $methodlist);
		}
	}

	/**
	* Return html code to include all needed jsolait libs
	*/
	function js_import_libs($jsolaitPath)
	{
		if ($jsolaitPath == '')
		{
			$jsolaitPath = '.';
		}
		$out = '<script type="text/javascript" src="'.$jsolaitPath.'/jsolait.js"></script>
';
// The following scripts are loaded by jsolait itself...
/*<script type="text/javascript" src="'.$jsolaitPath.'/lib/urllib.js"></script>';
		if ($protocol == 'jsonrpc')
		{
			$out .= '
<script type="text/javascript" src="'.$jsolaitPath.'/lib/jsonpc.js"></script>
';
		}
		else
		{
			$out .= '
<script type="text/javascript" src="'.$jsolaitPath.'/lib/xml.js"></script>
<script type="text/javascript" src="'.$jsolaitPath.'/lib/xmlrpc.js"></script>
';
		}*/
		return $out;
	}

	/**
	* Return js code providing a function to call a method of an xmlrpc server.
	* Note that method names should NOT contain dot characters
	* @todo catch run-time xmlrpc exceptions without a js alert
	*/
	function js_wrap_xmlrpc_method($method, $url, $liburl='', $protocol='xmrlpc')
	{
		//$method = str_replace('.', '_', $method);
		$out = 'function '.$method.' (args) {
	var '.$protocol.' = null;
	var '.$protocol.'_server = null;';
	/*if ($baseurl != '')
	{
		$out .= '
	jsolait.baseURL = '.$baseurl;
	}*/
	if ($liburl != '')
	{
		$out .= '
	jsolait.baseURI = "'.$liburl.'";';
	}
	$out .= '
	try {
		'.$protocol.' = imprt("'.$protocol.'");
		'.$protocol.'_server = new '.$protocol.'.ServiceProxy("'.$url.'", ["'.$method.'"]);
		try {
		   if (args == null) {	
		      return '.$protocol.'_server.'.$method.'();
         }
         else {
			   return '.$protocol.'_server.'.$method.'(args);
         }
		} catch (e) {
			alert(e);
		}
	} catch (e) {
		alert(e);
	}
}
';
		return $out;
	}

	/**
	* Return js code providing functions to call all methods of an xmlrpc server
	* Note that method names with a . will not work!
	* @param xmlrpcserver $server
	* @param array $methodlist list of methods to be wrapped into js (or null == expose all methods)
	* @param string $url url at which the xmlrpcserver responds
	* @param string $liburl url of the jsolait lib (see jsolait docs for more info)
	* @return string the js code snippet defining js functions that wrap web services
	*/
	function js_wrap_xmlrpc_server($server, $url, $liburl='', $methodlist=null)
	{
			if (is_a($server, 'jsonrpc_server'))
			{
				$protocol = 'jsonrpc';
			}
			else
			{
				$protocol = 'xmlrpc';
			}
			return js_wrap_dispatch_map($server->dmap, $url, $liburl, $methodlist, $protocol);
	}

	/**
	* Same functionality as 'js_wrap_xmlrpc_server', but needs only the server
	* dispatch map (and an optional server type) , not the server itself
	* @param array $dmap an xmlrpc server dispatch map
	* @param array $methodlist list of methods to be wrapped into js (or null = wrap all)
	* @param string $url url at which the xmlrpcserver responds
	* @param string $liburl url of the jsolait lib (see jsolait docs for more info)
	* @param string $protocol either 'xmlrpc' or 'jsonrpc'
	* @return string the js code snippet defining js functions that wrap web services
	*/
	function js_wrap_dispatch_map($dmap, $url, $liburl='', $methodlist=null, $protocol='xmlrpc')
	{
			$out = '';
			$out .= js_import_libs($liburl);
			if ($methodlist == null)
			{
				$methodlist = array_keys($dmap);
			}
			// be extra tolerant: a single method can be passed as string...
			else if (is_string($methodlist))
			{
			  $methodlist = array($methodlist);
			}

			$out .= '<script type="text/javascript">
';
			foreach($methodlist as $method)
			{
				if (array_key_exists($method, $dmap))
				{
					$out .= js_wrap_xmlrpc_method($method, $url, $liburl, $protocol);
				}
			}
			$out .= '</script>
';
			return $out;
	}

?>