<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


/**
 * @purpose provide a base Iterator implementation
 * @version $Revision$
 * $Id: iterator.class.php 493 2006-08-09 13:08:12Z kevbob $
 * 
 * It follows PHP5's iterator interface
 * 
 * call example:
 * $rs = mysql_query( "select blah blah blah" );
 * $it =& new FOF_ItemIterator( $rs );
 * $it->next();
 * while ( $it->valid() ) {
 * 	$row =& $it->current();
 *    // do something with $row
 * 
 *    // move to next row
 *    $it->next();
 * }
 * 
 *
 */

class FOF_Iterator
{
	
	var $key;
	var $current;
	var $valid;
		
	/**
	 * 
	 */
	function FOF_Iterator()
	{
		$this->valid = true;
		$this->key = -1;
		$this->current = null;
	}

   /**
    * Rewind the Iterator to the first element.
    * Similar to the reset() function for arrays in PHP
    * 
    * @return void
    */
   function rewind()
   {
      $this->key = -1;
   	$this->next();
   }
   
   /**
    * Return the current element.
    * Similar to the current() function for arrays in PHP
    * 
    * @return mixed current element from the collection
    */
   function current()
   {
   	return $this->current;
   }

   /**
    * Return the identifying key of the current element.
    * Similar to the key() function for arrays in PHP
    * 
    * @return mixed either an integer or a string
    */
   function key()
   {
   	return $this->key;
   }

   /**
    * Move forward to next element.
    * Similar to the next() function for arrays in PHP
    * 
    * @return void
    */
   function next()
   {
      $this->current = $this->_getNextItem();
      
      if ($this->current === false) {
         $this->valid = false;
      } else {
   	  $this->key++;
      }
   }
   
   /**
    * Check if there is a current element after calls to rewind() or next().
    * Used to check if we've iterated to the end of the collection
    * 
    * @return boolean FALSE if there's nothing more to iterate over
    */
   function valid()
   {
   	return $this->valid;
   }

   /**
    * get next item from somewhere.  This should be the only method
    * that needs to be overridden by subclasses.
    * 
    * @abstract
    * @return mixed or null
    */
   function _getNextItem()
   {
      die('FOF_Iterator::_getNextItem() must be overridden');  
   }
}

class FOF_ArrayIterator extends FOF_Iterator
{

   var $vals;
   
   function FOF_ArrayIterator( $inArray )
   {
      parent::FOF_Iterator();
      $this->vals = $inArray;
   }
   
   function _getNextItem()
   {
      // this function is called before the key is incremented
      // in the next method
      $nextKey = $this->key + 1;
      if ( $nextKey == count($this->vals) ) {
         return false;
      }
      else {
         return $this->vals[$nextKey];
      }

   }
}

/**
 * provide an empty iterator for methods that must return an iterator
 * but have no results to provide
 */
class FOF_EmptyIterator extends FOF_ArrayIterator
{
   function FOF_EmptyIterator()
   {
      $this->vals = array();
   }
}


