<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */

/**
 * @purpose provide helper functions to the views 
 *
 * @version $Revision$
 * $Id: view.class.php 546 2007-02-25 07:57:33Z kevbob $
 */


class View
{

   var $template;
   var $params = array();
   
   var $parent = null;
   var $contentType = 'text/html; charset=utf-8';
   
   function View( $template = null)
   {
   	$this->template = $template;

   }

   /**
    * render this view
    * 
    */
   function render()
   {
     
      if (is_null($this->parent)) {
				@header("Content-Type: text/html; charset=utf-8");
      }
      
      // populate fuction with view local variables	
      foreach ($this->params as $k => $v) {
      	$$k = $v;
      }
   
      // execute template
   	if (!is_null($this->template)) {
   		require $this->template;
   	}
   }

   function set($name, $value)
   {
   	$this->params[$name] = $value;
   }
   
   function setAll($arr)
   {
   	foreach ($arr as $k => $v) {
   		$this->params[$k] = $v;
   	}
   }
   
   function setParent($parentView)
   {
     $this->parent = $parentView;
   }
}


/**
 * implements Composite design pattern on views
 * a composite view will delegate rendering to one or more sub-views 
 * 
 */
class CompositeView
{
	
   var $views;

   function CompositeView()
   {
   	$this->views = array();
   }
   
   /**
    * add view to this composite
    * 
    * @param View $view
    * @return void
    */
   function addView( $view )
   {
   	$this->views[] = $view;
   }
   
   /**
    * render the composite view
    * 
    */
   function render()
   {
   	
      // we access the views array by key to avoid creating copies of the objects in php4
   	foreach (array_keys($this->views) as $key) {
   		$this->views[$key]->render();
   	}
   }
   
   function set($name, $value)
   {
   	die( "don't set on CompositeView, set on the child views directly" );
   }
   
   function setAll($arr)
   {
      die( "don't set on CompositeView, set on the child views directly" );
   }
}


/**
 * convenience class for rendering a view with a header above it, and a footer below it
 */
class LayoutView
{
   var $header;
   var $footer;
   var $main;	

   
	function LayoutView($header, $footer, $main )
   {
      $this->header = $header;
      $this->footer = $footer;
      $this->main = $main;
   }
   
   
   function render()
   {
    	$this->header->render();
      $this->main->render();
      $this->footer->render();
   }
}



/**
 * special view that actually does a client side redirect to another URI
 * 
 */
class RedirectView
{
	
   var $url;
	
   function RedirectView( $url)
   {
      $this->url = $url;
   }
   
   function render()
   {
      @header("Location: " . $this->url);
      print <<<EOB
<html>
 <head>
  <!-- <meta http-equiv="Refresh" content="0;url={$this->url}"> -->
 </head>
 <body>
  Redirect
 </body>
</html>
EOB;
   }
}

/**
 * return NO CONTENT http status code back to client
 * 
 * this is useful for post or other submissions where it is not necessary to return
 * any response at all
 * 
 * 
 */
class NoContentView
{
  
  function render()
  {
		@header("Status: 204 Yatta");    
  }
}
