<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */

/**
 * @purpose provide helper functions to the views 
 *
 * @version $Revision$
 * $Id: view_helper.php 521 2006-10-04 02:21:13Z kevbob $
 */


function fof_rss_age_string( $age )
{
   if ( $age == -1 ) {
      $agestr = "never";
      $agestrabbr = "&infin;";
   }
   else {
      $seconds = $age % 60;
      $minutes = $age / 60 % 60;
      $hours = $age / 60 / 60 % 24;
      $days = floor($age / 60 / 60 / 24);

      if ( $seconds ) {
         $agestr = "$seconds s";
         $agestrabbr = $seconds . "s";
      }

      if ( $minutes ) {
         $agestr = "$minutes m";
         
         $agestrabbr = $minutes . "m";
      }

      if ( $hours ) {
         $agestr = "$hours h";
           
         $agestrabbr = $hours . "h";
      }

      if ( $days ) {
         $agestr = "$days d";
         
         $agestrabbr = $days . "d";
      }
   }
   
   return array($agestr, $agestrabbr);
   
}


/**
 * parse HTTP request map and populate item search object
 * 
 * 
 */
function fof_request2search( $request )
{
   global $opts;
   
   $item_model =& FOF_ModelFactory::getItemModel();

   $searchObj = $item_model->getDefaultSearch();

   if (isset($request['feed'])
       and !empty($request['feed'])) {
      // cast from string to the proper type
      $feed_id = (int)$request['feed'];
      $searchObj->setFeed($feed_id);
   }
   
   if (isset($request['category'])
       and !empty($request['category'])) {
      $cat_id = (int)$request['category'];
      $searchObj->setCategory($cat_id);
   }
   
   if (isset($request['tag'])
       and !empty($request['tag'])) {
      $tag_id = (int)$request['tag'];
      $searchObj->setTag($tag_id);
   }
   
   if (isset($request['flag'])
       and !empty($request['flag'])) {
      switch ($request['flag']) {
        case 'all':
          $searchObj->unsetFlag();
          break;
        case 'unread':
          $searchObj->setFlag(FOF_UNREAD_FLAG);
          break;
        case 'saved':
          $searchObj->setFlag(FOF_SAVED_FLAG);
          break;
        default:
          $searchObj->setFlag(FOF_UNREAD_FLAG);
      }
   }
   
   if (isset($request['search'])
       and !empty($request['search'])) {
      $searchObj->setSearch($request['search']);
   }
   
   if (isset($request['start'])
       and !empty($request['start'])) {
      $startInt = (int) $request['start'];
      $searchObj->setOffset($startInt);
   }
   
   $searchObj->setLimit($opts->get('page.posts_per'));

   if (isset($request['date'])
       and !empty($request['date'])) {
      $dateVal = $request['date'];
      if ($dateVal == 'today') {
         $dateVal = date( "Y/m/d", time() - ($opts->get('global.time_offset') * 60 * 60) );
      }
      $searchObj->setDate($dateVal);
   }
   
   if (isset($request['order'])
       and !empty($request['order']) ) {
      $searchObj->setSortOrder($request['order']);
   }
   else {
   	$searchObj->setSortOrder('desc');
   }
   
   return $searchObj;
}



/**
 * marshal a search object into a query string
 * 
 * @param FOF_ItemSearch $search
 * @param bool $useOffset
 * @return string
 */
function fof_search2queryString( $search, $useOffset = true )
{
	
   global $opts;   

   $queryArr = array();
   $filter = $search->getFilter();
   
   
   // cat, feed, and tag are mutually exclusive filters
   // we only pass one of them through to the next page
   if ( $catFilter = $filter->findFilter('category_id') ) {
      array_push( $queryArr, sprintf('category=%d', $catFilter->getValue() ) );
   }
   elseif ($feedFilter = $filter->findFilter('feed_id')) {
      array_push( $queryArr, sprintf('feed=%d', $feedFilter->getValue() ) );
   }
   elseif ($tagFilter = $filter->findFilter('tag_id')) {
      array_push( $queryArr, sprintf('tag=%d', $tagFilter->getValue() ) );
   }
   
   
   if ($flagFilter = $filter->findFilter('i.flag') ) {
      
      if ($flagFilter->getValue() == FOF_SAVED_FLAG) {
      array_push( $queryArr, sprintf('flag=%s', 'saved' ) );
      }
   }
   else {
      array_push( $queryArr, sprintf('flag=%s', 'all' ) );
   }

   if ($dateFilter = $filter->findFilter('publish_time')) {

      array_push( $queryArr, sprintf('date=%s', $search->curDate ) );
   }

   if ($offset = $search->getOffset()
       and $useOffset) {
      
      array_push( $queryArr, sprintf('start=%d', $offset ) );	
   }
   
   if ($order = $search->getSortOrder()) {
      array_push( $queryArr, sprintf('order=%s', $order ) );  
      
   }

   if ($searchFilter = $filter->findFilter('search')) {
   	// need to find search term within filter
      $titleFilter = $searchFilter->findFilter('i.title');
      array_push( $queryArr, sprintf('search=%s', urlencode($titleFilter->getValue()) ) );  
   }

   return join('&amp;', $queryArr);
}

/**
 * render an informative title string describing the item search
 * 
 * @param FOF_ItemSearch $item_search
 * @return string
 */
function fof_view_title( $item_search )
{
   global $opts, $view;

   $cat_model = FOF_ModelFactory::getCategoryModel();
   $feed_model = FOF_ModelFactory::getFeedModel();

   $title = "fofRedux";

   $filter = $item_search->getFilter();

   if ( $dateFilter = $filter->findFilter('publish_time') ) {
   	
      // extract date from range filter returned
      // $low value is stored as an epoch timestamp, convert to string:
      $whendate = date( "Y/m/d", $dateFilter->getLowValue() - ($opts->get('global.time_offset') * 60 * 60) );

      
      $title .= ' - ' . $whendate;
   }
   
   if ( $catFilter = $filter->findFilter('category_id'))
   {
      $category = $cat_model->fetch($catFilter->getValue());
      
      $title .= ' - ' . htmlspecialchars($category->getTitle());
   }
   
   if ($feedFilter = $filter->findFilter('feed_id')) {
      $feed = $feed_model->fetch($feedFilter->getValue());
      $title .=' - ' . htmlspecialchars($feed->getTitle());
   }
   if($item_search->getOffSet() > 0) {
   	
      $start = $item_search->getOffSet();
      $limit = $opts->get('page.posts_per');
      $title .= " - items $start to " . ($start + $limit);
   }
   
   if ($flagFilter = $filter->findFilter('i.flag') ) {

      if ($flagFilter->getValue() == FOF_SAVED_FLAG) {
         $title .= ' - saved items';
      }
      else {
      	$title .= ' - unread items';
      }
   }
   else if($searchFilter = $filter->findFilter('search'))
   {
      // need to find search term within filter
      $titleFilter = $searchFilter->findFilter('i.title');
      
      $title .= ' - search: ' . htmlspecialchars($titleFilter->getValue());
   }
   else
   {
      $title .= ' - all items';
   }

   return $title;
}


/** 
* Creates a table of current categories.
*
* @todo fix the count() to display categories 
* that have no feed items.
*/
function fof_category_list_HTML()
{
   $cat_model = FOF_ModelFactory::getCategoryModel();
   
   $cat_iter = $cat_model->getCategories();
   $cat_iter->rewind();
   $html = '<table>';

   while ( $cat_iter->valid() ) {
      
      $category = $cat_iter->current();
      $cat_iter->next();
      
      $category_id = $category->getId();
      $category_title = $category->getTitle();
      
      $feed_count = $cat_model->getFeedCount($category);
      
      if ( ($cat_iter->key() % 2) == 0) {
         $html .= '<tr class="odd-row">';
      }
      else {
         $html .= '<tr class="even-row">';
      }
      
      $html .= <<<EOB
 <td><a href="view.php?category=$category_id" class="category_link">$category_title</a></td>  
 <td class="category_feed_count">$feed_count</td>
 <td><a href="rss.php?category=$category_id" class="rssfeed_link">rss</a></td>
 <td><a href="edit.php?category=$category_id" class="category_edit">edit</a></td>
 <td class="category_delete"><a href="delete.php?category=$category_id" onclick="return confirm('Are you sure you want to delete the $category_title category?');">delete</a></td>
</tr>
EOB;
   }
   $html .= '</table>';
   return $html;
}


function fof_category_select_HTML($select_name, $selected_category=NULL)
{

   $html = '<select name="'.$select_name.'" id="'.$select_name.'">';

   $cat_model =& FOF_ModelFactory::getCategoryModel();

   // get iterator of categories from the model
   $cat_iter =& $cat_model->getCategories();
   
   $cat_iter->rewind();
   
   while ( $cat_iter->valid() ) {
      $category = $cat_iter->current();
      $cat_iter->next();

      if ($category->getId() === $selected_category) {
         $html .= "<option value=\"".$category->getId()."\" selected=\"selected\">".$category->getTitle()."</option>";
      }
      else {
         $html .= "<option value=\"".$category->getId()."\">".$category->getTitle()."</option>";
  
      }
   }

   $html .= '</select>';

   return $html;
}




/**
 * format an html string for the feed name and version
 * including relevent url links
 * 
 * @param FOF_FeedExt $row
 * @return array
 */
function fof_render_feed_link(&$feed)
{
   $link = htmlspecialchars($feed->getLink());
   $title = htmlspecialchars($feed->getTitle());
   $url = htmlspecialchars($feed->getUrl());

   $description = $feed->getDescription();
   if (is_null($description))
      $description = '?';
   $feed_type = $feed->getRssVersion();
   if (is_null($feed_type))
      $feed_type = '?';

   $description = htmlspecialchars($description);
   $feed_type = htmlspecialchars($feed_type);   
   
   $string = "<a class=\"feed_link\" href=\"$link\" title=\"$description\">$title</a>" .
        "<a class=\"feed_type\" href=\"$url\">($feed_type)</a>";

   return $string;
}


/**
 * format the navigation links for view.php
 *
 * @param FOF_ItemSearch $search
 * @return string
 */
function fof_get_nav_links( $item_search )
{
   global $opts, $view;
   
   $item_model = FOF_ModelFactory::getItemModel();
   
   $string = "";
   
   $filter = $item_search->getFilter();
   
   // create previous date/next date links
   if($dateFilter = $filter->findFilter('publish_time')) {

      $prev_day_search = $item_search->getPrevDate();
      $prev_day_search->setOffSet(0);
      
      if ($item_model->search($prev_day_search, true)) {
         $prev_filter = $prev_day_search->getFilter();
         $prevDateFilter = $prev_filter->findFilter('publish_time');
         $prev_string = date( "Y/m/d", $prevDateFilter->getLowValue() - ($opts->get('global.time_offset') * 60 * 60) );
         $string .= sprintf('<a href="view.php?%s&amp;view=%s">[&laquo; %s]</a> ',
                             fof_search2queryString($prev_day_search,false),$view,$prev_string);
      }

      $next_day_search = $item_search->getNextDate();
      $next_day_search->setOffSet(0);
      
      if ($item_model->search($next_day_search, true)) {
         $next_filter = $next_day_search->getFilter();
         $nextDateFilter = $next_filter->findFilter('publish_time');
         $next_string = date( "Y/m/d", $nextDateFilter->getLowValue() - ($opts->get('global.time_offset') * 60 * 60) );
         $string .= sprintf('<a href="view.php?%s&amp;view=%s">[%s &raquo;]</a> ',
                             fof_search2queryString($next_day_search,false),$view,$next_string);
      }
   }

   $prev_offset_search = $item_search->getPrevOffset();

   if ($prev_offset_search->getOffset() >= 0
       and $item_model->search($prev_offset_search, true)) {

      $string .= sprintf('<a href="view.php?%s&amp;view=%s">[&laquo; previous %d]</a> ',
                         fof_search2queryString($prev_offset_search),$view,$prev_offset_search->getLimit());

   }

   $next_offset_search = $item_search->getNextOffset();

   if ($item_model->search($next_offset_search, true)) {

      $string .= sprintf('<a href="view.php?%s&amp;view=%s">[next %d &raquo;]</a> ',
                         fof_search2queryString($next_offset_search),$view,$next_offset_search->getLimit());

   }

   return $string;
}


/**
 * remove malicious tags from item contents
 * code attributed to http://www.php.net/manual/en/function.strip-tags.php#36574
 *
 * @param string $text    raw content string
 * @return string
 */
function fof_sanitize_content( $text )
{

   // add a space before and after every tag
//   $text = preg_replace('/</',' <',$text);
//   $text = preg_replace('/>/','> ',$text);

   // the filter routine is run as many times as necesary to ensure all
   // bad tags/attributes are removed
   while ( true ) {
      // save a copy before we start
      $origText = $text;

      // remove malicious tags completely, including their content
      $text = preg_replace('/<((no)?script|body|link|embed|style|frameset|i?frame)[^>]*>[^<]*<\/\\1>/i', '', $text);

      // do same as above on tags without closing tags
      $text = preg_replace('/<[\/]*((no)?script|body|link|embed|style|frameset|i?frame)[^>]*>/i', '', $text);

      // balance tags first
      $text = fof_balanceTags( $text );

      // remove evil attributes from remaining tags
      // the pattern limits it it tags containing an equal sign
      $text = preg_replace_callback('/<[^>]+=[^>]+>/', "fof_removeEvilAttributes", $text);

      // exit while loop if the above operations did not change the text at all
      if ( $text === $origText ) break;
   }

   // condense sequences of spaces to just one
   $text = preg_replace('/ +/',' ',$text);

   return $text;
}

/**
 * remove evil attributes from the preg_match array
 * this is intended to be called from preg_replace_callback()
 * that's why we have to receive a match array not a simple string
 * 
 * @param array $matches
 * @return string
 */
function fof_removeEvilAttributes($matches)
{
   $tagSource = $matches[0];
   /**
    * Disallow these attributes/prefix within a tag
    */
   $stripAttrib = 'onclick|ondblclick|onmousedown|onmouseup|onmouseover|'.
                  'onmousemove|onmouseout|onkeypress|onkeydown|onkeyup|class|style';

   $stripAttrib = "' ($stripAttrib)=\"[^\"]+\"'i";
   $tagSource = stripslashes($tagSource);
   $matches[] = $tagSource;
   $tagSource = preg_replace($stripAttrib, '', $tagSource);
      $matches[] = $tagSource;
   $tagSource = preg_replace('/"javascript:/i', '"bad-javascript-bad:', $tagSource);
      $matches[] = $tagSource;

   return $tagSource;
}

function fof_normalize_content_urls($content, $start_url )
{

   // get root url from start url (directory, with the filename chopped off)
   $base_url = dirname( $start_url );
   
   // get root url from base/start
   $url_parsed = parse_url($base_url);

   $root_url = sprintf( '%s://%s', $url_parsed['scheme'], $url_parsed['host'] ); 

   // get all image and hyperlink URLs from a content string
   $matches = null;
   preg_match_all(
            '/<(img|a) [^>]*(src|href)="([^"]+)"[^>]*>/i',
            $content,
            $matches );
   
   // loop through the URIs found in the content, check if we should reformat them
   // then replace them back into the original content
   // we unique the URI list because preg_replace will take care of replacing
   // all instances of the URI
   foreach (array_unique($matches[3]) as $match_url ) {
      
      // create full URL for the input URI
      $new_url = fof_url_make_full( $match_url, $base_url, $root_url);
      
      if ( $new_url !== $match_url ) {
         // replace original url with the new full path one

         $content = preg_replace( "|\"$match_url\"|", "\"$new_url\"", $content );
      }
   }
   
   return $content;
}

/**
 * reformat a (possibly) relative URL/URI with the fully qualified URL path
 * 
 * @param string $uri
 * @param string $base_url
 * @param string $root_url
 * @return string
 */
function fof_url_make_full( $uri, $base_url, $root_url )
{
    $new_url = $uri;
    $matches = null;
    
    if ( preg_match('/^http/',$uri) ) {
       // do nothing, url is fully qualified.
    }
    elseif ( preg_match('/^\/(.+)/', $uri, $matches) ) {
    	
       //url starts with a slash so it is relative to the root_url
       $new_url = sprintf( "%s/%s", $root_url, $matches[1] );
    }
    else {
    	
       //url must now be relative to base_url
       $new_url = sprintf( "%s/%s", $base_url, $uri );
    }

   return $new_url;
}


/*
 balanceTags

 Balances Tags of string using a modified stack.

 @param text      Text to be balanced
 @return          Returns balanced text
 @author          Leonard Lin (leonard@acm.org)
 @version         v1.1
 @date            November 4, 2001
 @license         GPL v2.0
 @notes
 @changelog
         ---  Modified by Scott Reilly (coffee2code) 02 Aug 2004
             1.2  ***TODo*** Make better - change loop condition to $text
             1.1  Fixed handling of append/stack pop order of end text
                  Added Cleaning Hooks
             1.0  First Version
*/
function fof_balanceTags($text, $is_comment = 0) {

//    if (get_settings('use_balanceTags') == 0) {
//        return $text;
//    }

   $tagstack = array(); $stacksize = 0; $tagqueue = ''; $newtext = '';

   # WP bug fix for comments - in case you REALLY meant to type '< !--'
   $text = str_replace('< !--', '<    !--', $text);
   # WP bug fix for LOVE <3 (and other situations with '<' before a number)
   $text = preg_replace('#<([0-9]{1})#', '&lt;$1', $text);

   $regex = null;
   while (preg_match("/<(\/?\w*)\s*([^>]*)>/",$text,$regex)) {
      $newtext .= $tagqueue;

      $i = strpos($text,$regex[0]);
      $l = strlen($regex[0]);

      // clear the shifter
      $tagqueue = '';
      // Pop or Push
      if (isset($regex[1][0])
          and $regex[1][0] == "/") { // End Tag
         $tag = strtolower(substr($regex[1],1));
         // if too many closing tags
         if($stacksize <= 0) {
            $tag = '';
            //or close to be safe $tag = '/' . $tag;
         }
         // if stacktop value = tag close value then pop
         else if ($tagstack[$stacksize - 1] == $tag) { // found closing tag
            $tag = '</' . $tag . '>'; // Close Tag
            // Pop
            array_pop ($tagstack);
            $stacksize--;
         } else { // closing tag not at top, search for it
            for ($j=$stacksize-1;$j>=0;$j--) {
               if ($tagstack[$j] == $tag) {
                  // add tag to tagqueue
                  for ($k=$stacksize-1;$k>=$j;$k--){
                     $tagqueue .= '</' . array_pop ($tagstack) . '>';
                     $stacksize--;
                  }
                  break;
               }
            }
            $tag = '';
         }
      } else { // Begin Tag
         $tag = strtolower($regex[1]);

         // Tag Cleaning

         // If self-closing or '', don't do anything.
         if((substr($regex[2],-1) == '/') || ($tag == '')) {
         }
         // ElseIf it's a known single-entity tag but it doesn't close itself, do so
         elseif ($tag == 'br' || $tag == 'img' || $tag == 'hr' || $tag == 'input') {
            $regex[2] .= '/';
         } else {    // Push the tag onto the stack
            // If the top of the stack is the same as the tag we want to push, close previous tag
            if (($stacksize > 0) && ($tag != 'div') && ($tagstack[$stacksize - 1] == $tag)) {
               $tagqueue = '</' . array_pop ($tagstack) . '>';
               $stacksize--;
            }
            $stacksize = array_push ($tagstack, $tag);
         }

         // Attributes
         $attributes = $regex[2];
         if($attributes) {
            $attributes = ' '.$attributes;
         }
         $tag = '<'.$tag.$attributes.'>';
         //If already queuing a close tag, then put this tag on, too
         if ($tagqueue) {
            $tagqueue .= $tag;
            $tag = '';
         }
      }
      $newtext .= substr($text,0,$i) . $tag;
      $text = substr($text,$i+$l);
   }

   // Clear Tag Queue
   $newtext .= $tagqueue;

   // Add Remaining text
   $newtext .= $text;

   // Empty Stack
   while($x = array_pop($tagstack)) {
      $newtext .= '</' . $x . '>'; // Add remaining tags to close
   }

   // WP fix for the bug with HTML comments
   $newtext = str_replace("< !--","<!--",$newtext);
   $newtext = str_replace("<    !--","< !--",$newtext);

   return $newtext;
}

