<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * fofRedux Copyright (C) 2005 Andrew Turner
 * nilspace@users.sourceforge.net - http://highearthorbit.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


require_once('tests_common.inc.php');


/**
 *
 * @author Kevin Bennett <kev@drule.org>
 */
class ContentURLRewriteTC extends UnitTestCase
{

	/**
	 * test content with no HTML markup, it should pass through unchanged
	 */
	function testPlainContent()
	{
		$input = <<<EOF
this is some input
ha ha
EOF;

		$this->assertEqual(fof_sanitize_content($input), $input);
	}

	function testRelativeImage1()
	{
	$url = "http://joeshaw.org/2005/11/27/188";
	$input = <<<EOF
<div><a href="/images/jimmy-joanna.jpg">
	<img src="/images/jimmy-joanna-thumb.jpg"
	 alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></a>
	<div>dorks</div>
</div>
	<div><a href="http://joeshaw.org/images/williamsburg.jpg">
	<img src="/images/williamsburg-thumb.jpg" alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></a>
	<div>
EOF;
	
			$output = <<<EOF
<div><a href="http://joeshaw.org/images/jimmy-joanna.jpg">
	<img src="http://joeshaw.org/images/jimmy-joanna-thumb.jpg"
	 alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></a>
	<div>dorks</div>
</div>
	<div><a href="http://joeshaw.org/images/williamsburg.jpg">
	<img src="http://joeshaw.org/images/williamsburg-thumb.jpg" alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></a>
	<div>
EOF;
		  $retval = fof_normalize_content_urls($input, $url);
		  $this->assertEqual($retval, $output);
	}
   
   
      function testRelativeMixedCase()
   {
   $url = "http://joeshaw.org/2005/11/27/188";
   $input = <<<EOF
<div><A HREF="/images/jimmy-joanna.jpg">
   <img src="/images/jimmy-joanna-thumb.jpg"
    alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></A>
   <div>dorks</div>
</div>
   <div><a href="http://joeshaw.org/images/williamsburg.jpg">
   <img src="/images/williamsburg-thumb.jpg" alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></a>
   <div>
EOF;
   
         $output = <<<EOF
<div><A HREF="http://joeshaw.org/images/jimmy-joanna.jpg">
   <img src="http://joeshaw.org/images/jimmy-joanna-thumb.jpg"
    alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></A>
   <div>dorks</div>
</div>
   <div><a href="http://joeshaw.org/images/williamsburg.jpg">
   <img src="http://joeshaw.org/images/williamsburg-thumb.jpg" alt="[A photo by Joe Shaw]" title="[A photo by Joe Shaw]"/></a>
   <div>
EOF;
        $retval = fof_normalize_content_urls($input, $url);
        $this->assertEqual($retval, $output);
   }
   
}


// if we are not included in another test runner
// run above test case and output an HTML report
if (!defined('FOF_TEST_REPORT')) {
	$test = new ContentURLRewriteTC();

	$test->run(new HtmlReporter());

}

