<?php
/*
 * This file is part of fofRedux - http://fofredux.sf.net/
 *  a fork of the Feed on Feeds project - http://feedonfeeds.com
 *
 * fofRedux Copyright (C) 2005 Andrew Turner
 * nilspace@users.sourceforge.net - http://highearthorbit.com
 *
 * Distributed under the GPL - see LICENSE
 *
 */


require_once('tests_common.inc.php');


/**
 *
 * @author Kevin Bennett <kev@drule.org>
 */
class SanitizeContentTestCase extends UnitTestCase
{

	/**
	 * test content with no HTML markup, it should pass through unchanged
	 */
	function testPlainContent()
	{
		$input = <<<EOF
this is some input
ha ha
EOF;

		$this->assertEqual(fof_sanitize_content($input), $input);
	}

	/**
	 * test where content has a closing table tag with no open tag
	 */
	function testCloseTableTag()
	{
		$input = <<<EOF
this is has a closing table tag </table> blah
EOF;

		$expResult = <<<EOF
this is has a closing table tag blah
EOF;

		$this->assertEqual($expResult, fof_sanitize_content($input));
	}

	function testContainsGoodHTML()
	{
		$input = <<<EOF
   <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
   set up WordPress and is <a href="http://off.net/diary/2005/12/06/coming-up-roses/">writting funny</a> again.</p>
EOF;

		$output = fof_sanitize_content($input);
		$input = preg_replace('/ +/',' ',$input);

		$this->assertEqual($output, $input);
	}

	function testContainsGoodHTML2()
	{

		$input = <<<EOF
<table border="3" cellpadding="2" cellspacing="0" bgcolor="#ffffef">
<tr>
<td align="right">Name</td>
 <td><b>Bella</b></td><td align="right">ID</td> <td><b>4702</b>
 </td><td rowspan="5" width="160">
 <a href="http://www.longmonthumane.org/adopt/display.php?name=Bella&id=4702" target="_self">
 <img src="http://www.longmonthumane.org/adopt/animals/bella_4702s.jpg" alt="Bella" width="160" height="120" border="0" /></a></td></tr>
 <tr><td align="right">Breed</td> <td colspan="2"><b>Labrador Retriever Mix</b></td>
 <td align="center"><a href="cg.html" target="_self"><img src="graphics/blu1.gif" alt="blu" height="16" width="16" border="0" /></a></td></tr><tr>
 <td align="right">Gender</td> <td><b>Female</b></td><td align="right">Weight</td> <td><b>85 lb</b>
 </td></tr><tr><td align="right">Age</td> <td colspan="3"><b>5 years</b></td></tr><tr>
 <td align="right">Color</td> <td colspan="3"><b>Black with White</b></td></tr></table>
EOF;

		$output = fof_sanitize_content($input);
		$input = preg_replace('/ +/',' ',$input);

		$this->assertEqual($output, $input);

	}

	function testContainsScriptTag()
	{

		$input = <<<EOF
<script type="text/javascript"> alert("foo");
</script>
 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$expResult = <<<EOF

 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$output = fof_sanitize_content($input);

		$this->assertEqual($expResult,$output);
	}

	function testContainsScriptAttr()
	{
		$input = <<<EOF
 <p>I am really happy my friend <a onClick="dobadstuff();" href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$expResult = <<<EOF
 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$output = fof_sanitize_content($input);

		$this->assertEqual($expResult,$output);
	}


	function testContainsEmbed()
	{

		$input = <<<EOF
 <p>I am really happy my friend <EMBED
SRC="../graphics/sounds/1812over.mid"
HEIGHT=60 WIDTH=144><a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$expResult = <<<EOF
 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$output = fof_sanitize_content($input);

		$this->assertEqual($expResult,$output);
	}

	function testContainsLink()
	{
		$input = <<<EOF
<link rel="alternate" href="http://www.fark.com/fark.rss" type="application/rss+xml" title="Fark.com RSS Feed">
 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$expResult = <<<EOF

 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$output = fof_sanitize_content($input);

		$this->assertEqual($expResult,$output);

	}

	function testContainsStyleTag()
	{

		$input = <<<EOF
<STYLE TYPE="text/css">
   H1.artdeco { color: #00FFFF }
  </STYLE>
 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$expResult = <<<EOF

 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;


		$output = fof_sanitize_content($input);

		$this->assertEqual($expResult,$output);

	}

	function testContainsStyleAttr()
	{
			$input = <<<EOF
 <p>I am really happy my friend <a style="font-color:red;" href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;

		$expResult = <<<EOF
 <p>I am really happy my friend <a href="http://off.net/diary/">phil</a>
 set up WordPress and is funny again.</p>
EOF;


		$output = fof_sanitize_content($input);

		$this->assertEqual($expResult,$output);

	}

	/**
	 * @param string $text
	 * @return void
	 */
	function printer($text) {
		print "\n<pre>".$text."</pre><br/>\n";
	}
}


// if we are not included in another test runner
// run above test case and output an HTML report
if (!defined('FOF_TEST_REPORT')) {
	$test = new SanitizeContentTestCase();

	$test->run(new HtmlReporter());

}

